@@ -7,140 +7,121 @@ use Alzabo;
 
 use Tie::IxHash;
 
-use Params::Validate qw( :all );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
+#use fields qw( name table attributes nullable definition sequenced default );
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.22 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
 sub table
 {
-    $_[0]->{table};
+    my Alzabo::Column $self = shift;
+
+    return $self->{table};
 }
 
 sub name
 {
-    $_[0]->{name};
+    my Alzabo::Column $self = shift;
+
+    return $self->{name};
 }
 
+*null = \&nullable;
 sub nullable
 {
-    $_[0]->{nullable};
+    my Alzabo::Column $self = shift;
+
+    return $self->{nullable};
 }
 
 sub attributes
 {
-    return keys %{ $_[0]->{attributes} };
+    my Alzabo::Column $self = shift;
+
+    return keys %{ $self->{attributes} };
 }
 
 sub has_attribute
 {
-    my $self = shift;
-    my %p = validate( @_, { attribute => { type => SCALAR },
-			    case_sensitive => { type => SCALAR,
-						default => 0 } } );
+    my Alzabo::Column $self = shift;
+    my %p = @_;
 
-    if ( $p{case_sensitive} )
-    {
-        return exists $self->{attributes}{ $p{attribute} };
-    }
-    else
-    {
-        return 1 if grep { lc $p{attribute} eq lc $_ } keys %{ $self->{attributes} };
-    }
+    my $att = $p{case_sensitive} ? $p{attribute} : lc $p{attribute};
+
+    return exists $self->{attributes}{$att};
 }
 
 sub type
 {
-    $_[0]->definition->type;
+    my Alzabo::Column $self = shift;
+
+    return $self->definition->type;
 }
 
 sub sequenced
 {
-    $_[0]->{sequenced};
+    my Alzabo::Column $self = shift;
+
+    return $self->{sequenced};
 }
 
 sub default
 {
-    $_[0]->{default};
+    my Alzabo::Column $self = shift;
+
+    return $self->{default};
 }
 
 sub length
 {
-    $_[0]->definition->length;
+    my Alzabo::Column $self = shift;
+
+    return $self->definition->length;
 }
 
 sub precision
 {
-    $_[0]->definition->precision;
+    my Alzabo::Column $self = shift;
+
+    return $self->definition->precision;
 }
 
 sub definition
 {
-    $_[0]->{definition};
+    my Alzabo::Column $self = shift;
+
+    return $self->{definition};
 }
 
 sub is_primary_key
 {
-    $_[0]->table->column_is_primary_key($_[0]);
+    my Alzabo::Column $self = shift;
+
+    return $self->table->column_is_primary_key($self);
 }
 
 sub is_numeric
 {
-    $_[0]->table->schema->rules->type_is_numeric($_[0]);
-}
+    my Alzabo::Column $self = shift;
 
-sub is_integer
-{
-    $_[0]->table->schema->rules->type_is_integer($_[0]);
-}
-
-sub is_floating_point
-{
-    $_[0]->table->schema->rules->type_is_floating_point($_[0]);
+    return $self->table->schema->rules->type_is_numeric( $self->type );
 }
 
 sub is_character
 {
-    $_[0]->table->schema->rules->type_is_char($_[0]);
-}
+    my Alzabo::Column $self = shift;
 
-sub is_date
-{
-    $_[0]->table->schema->rules->type_is_date($_[0]);
-}
-
-sub is_datetime
-{
-    $_[0]->table->schema->rules->type_is_datetime($_[0]);
-}
-
-sub is_time
-{
-    $_[0]->table->schema->rules->type_is_time($_[0]);
+    return $self->table->schema->rules->type_is_char( $self->type );
 }
 
 sub is_blob
 {
-    $_[0]->table->schema->rules->type_is_blob($_[0]);
+    my Alzabo::Column $self = shift;
+
+    return $self->table->schema->rules->type_is_blob( $self->type );
 }
 
-sub generic_type
-{
-    my $self = shift;
-
-    foreach my $type ( qw( integer floating_point character date datetime time blob ) )
-    {
-        my $method = "is_$type";
-        return $type if $self->$method();
-    }
-
-    return 'unknown';
-}
-
-sub comment { $_[0]->{comment} }
-
 __END__
 
 =head1 NAME
@@ -267,20 +248,6 @@ table's primary key.
 A boolean value indicating whether the column is a numeric type
 column.
 
-=head2 is_integer
-
-=head3 Returns
-
-A boolean value indicating whether the column is a numeric type
-column.
-
-=head2 is_floating_point
-
-=head3 Returns
-
-A boolean value indicating whether the column is a numeric type
-column.
-
 =head2 is_character
 
 =head3 Returns
@@ -288,63 +255,12 @@ column.
 A boolean value indicating whether the column is a character type
 column.
 
-This is true only for any columns which are defined to hold I<text>
-data, regardless of size.
-
-=head2 is_date
-
-=head3 Returns
-
-A boolean value indicating whether the column is a date type column.
-
-=head2 is_datetime
-
-=head3 Returns
-
-A boolean value indicating whether the column is a datetime type
-column.
-
-=head2 is_time
-
-=head3 Returns
-
-A boolean value indicating whether the column is a time type column.
-
 =head2 is_blob
 
 =head3 Returns
 
 A boolean value indicating whether the column is a blob column.
 
-This is true for any columns defined to hold binary data, regardless
-of size.
-
-=head2 generic_type
-
-=head3
-
-This methods returns one of the following strings:
-
-=over 4
-
-=item integer
-
-=item floating_point
-
-=item character
-
-=item date
-
-=item datetime
-
-=item time
-
-=item blob
-
-=item unknown
-
-=back
-
 =head2 definition
 
 The definition object is very rarely of interest.  Use the
@@ -356,12 +272,6 @@ type.
 The L<C<Alzabo::ColumnDefinition>|Alzabo::ColumnDefinition> object
 which holds this column's type information.
 
-=head2 comment
-
-=head3 Returns
-
-The comment associated with the column object, if any.
-
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
