@@ -4,15 +4,10 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Create;
-use Alzabo::Exceptions ( abbr => 'params_exception' );
-
-use Params::Validate qw( :all );
-Params::Validate::validation_options
-    ( on_fail => sub { params_exception join '', @_ } );
 
 use base qw(Alzabo::Column);
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.25 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -30,30 +25,11 @@ sub new
 
 sub _init
 {
-    my $self = shift;
-
-    my %p =
-        validate( @_, { table => { isa => 'Alzabo::Table' },
-                        name  => { type => SCALAR },
-                        null  => { optional => 1 },
-                        nullable => { optional => 1 },
-                        type  => { type => SCALAR,
-                                   optional => 1 },
-                        attributes => { type => ARRAYREF,
-                                        optional => 1 },
-                        default    => { type => UNDEF | SCALAR,
-                                        optional => 1 },
-                        sequenced  => { optional => 1 },
-                        length => { type => UNDEF | SCALAR,
-                                    optional => 1 },
-                        precision  => { type => UNDEF | SCALAR,
-                                        optional => 1 },
-                        definition => { isa => 'Alzabo::Create::ColumnDefinition',
-                                        optional => 1 },
-                        comment => { type => UNDEF | SCALAR,
-                                     default => '' },
-		  } );
+    my Alzabo::Create::Column $self = shift;
+    my %p = @_;
 
+    Alzabo::Exception::Params->throw( error => 'No table provided' )
+	unless $p{table};
     $self->set_table( $p{table} );
 
     $self->set_name( $p{name} );
@@ -66,12 +42,9 @@ sub _init
     }
     else
     {
-	$self->set_definition
-            ( Alzabo::Create::ColumnDefinition->new
-                  ( owner => $self,
-                    type => $p{type},
-                  )
-            );
+	$self->set_definition( Alzabo::Create::ColumnDefinition->new( owner => $self,
+								      type => $p{type},
+								    ) );
     }
 
     my %attr;
@@ -84,31 +57,22 @@ sub _init
     $self->set_default( $p{default} );
 
     $self->set_length( length => $p{length}, precision => $p{precision} );
-
-    $self->set_comment( $p{comment} );
 }
 
 sub set_table
 {
-    my $self = shift;
+    my Alzabo::Create::Column $self = shift;
 
-    validate_pos( @_, { isa => 'Alzabo::Create::Table' } );
     $self->{table} = shift;
 }
 
 sub set_name
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $name = shift;
 
-    params_exception "Column $name already exists in table"
-        if $self->table->has_column($name);
-
     my $old_name = $self->{name};
     $self->{name} = $name;
-
     eval
     {
 	$self->table->schema->rules->validate_column_name($self);
@@ -116,8 +80,7 @@ sub set_name
     if ($@)
     {
 	$self->{name} = $old_name;
-
-        rethrow_exception($@);
+	$@->rethrow;
     }
 
     $self->table->register_column_name_change( column => $self,
@@ -125,17 +88,16 @@ sub set_name
 	if $old_name;
 }
 
+*set_null = \&set_nullable;
 sub set_nullable
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $n = shift;
 
-    params_exception "Invalid value for nullable attribute: $n"
+    Alzabo::Exception::Params->throw( error => "Invalid value for nullable attribute: $n" )
 	unless $n eq '1' || $n eq '0';
 
-    params_exception "Primary key column cannot be nullable"
+    Alzabo::Exception::Params->throw( error => "Primary key column cannot be nullable" )
 	if $n eq '1' && $self->is_primary_key;
 
     $self->{nullable} = $n;
@@ -143,26 +105,23 @@ sub set_nullable
 
 sub set_default
 {
-    my $self = shift;
+    my Alzabo::Create::Column $self = shift;
 
-    validate_pos( @_, { type => UNDEF | SCALAR } );
     $self->{default} = shift;
 }
 
 sub set_length
 {
-    my $self = shift;
+    my Alzabo::Create::Column $self = shift;
 
     $self->{definition}->set_length(@_);
 }
 
 sub set_attributes
 {
-    my $self = shift;
+    my Alzabo::Create::Column $self = shift;
 
-    validate_pos( @_, ( { type => SCALAR } ) x @_ );
-
-    %{ $self->{attributes} } = ();
+    tie %{ $self->{attributes} }, 'Tie::IxHash';
 
     foreach (@_)
     {
@@ -172,9 +131,7 @@ sub set_attributes
 
 sub add_attribute
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $attr = shift;
 
     $attr =~ s/^\s+//;
@@ -188,74 +145,29 @@ sub add_attribute
 
 sub delete_attribute
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $attr = shift;
 
-    params_exception "Column " . $self->name . " doesn't have attribute $attr"
+    Alzabo::Exception::Params->throw( error => "Column " . $self->name . " doesn't have attribute $attr" )
 	unless exists $self->{attributes}{$attr};
 
     delete $self->{attributes}{$attr};
 }
 
-sub alter
-{
-    my $self = shift;
-    $self->{definition}->alter(@_);
-
-    # this will force them to go through the rules code again.
-    # Attributes that don't work with the new type are silently
-    # discarded.
-    foreach ( $self->attributes )
-    {
-	$self->delete_attribute($_);
-	eval { $self->add_attribute($_) };
-    }
-}
-
 sub set_type
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $t = shift;
 
     $self->{definition}->set_type($t);
-
-    # this will force them to go through the rules code again.
-    # Attributes that don't work with the new type are silently
-    # discarded.
-    foreach ( $self->attributes )
-    {
-	$self->delete_attribute($_);
-	eval { $self->add_attribute($_) };
-    }
-
-    if ( $self->length )
-    {
-	eval { $self->set_length( length => $self->length,
-				  precision => $self->precision ) };
-	if ($@)
-	{
-	    eval { $self->set_length( length => $self->length, precision => undef ) };
-	    if ($@)
-	    {
-		$self->set_length( length => undef,
-				   precision => undef );
-	    }
-	}
-    }
 }
 
 sub set_sequenced
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Column $self = shift;
     my $s = shift;
 
-    params_exception "Invalid value for sequenced attribute: $s"
+    Alzabo::Exception::Params->throw( error => "Invalid value for sequenced attribute: $s" )
 	unless $s eq '1' || $s eq '0';
 
     $self->table->schema->rules->validate_sequenced_attribute($self)
@@ -266,25 +178,12 @@ sub set_sequenced
 
 sub set_definition
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::ColumnDefinition' } );
+    my Alzabo::Create::Column $self = shift;
     my $d = shift;
 
     $self->{definition} = $d;
 }
 
-sub set_comment { $_[0]->{comment} = defined $_[1] ? $_[1] : '' }
-
-sub save_current_name
-{
-    my $self = shift;
-
-    $self->{last_instantiated_name} = $self->name;
-}
-
-sub former_name { $_[0]->{last_instantiated_name} }
-
 __END__
 
 =head1 NAME
@@ -346,10 +245,6 @@ One of either ...
 
 =item * definition => C<Alzabo::Create::ColumnDefinition> object
 
-=item * comment => $comment
-
-An optional comment.
-
 =back
 
 =head3 Returns
@@ -362,24 +257,6 @@ L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
 
 =for pod_merge type
 
-=head2 alter
-
-This method allows you to change a column's type, length, and
-precision as a single operation and should be instead of calling
-C<set_type> followed by C<set_length>.
-
-=head3 Parameters
-
-=over 4
-
-=item * type => $type
-
-=item * length => $length (optional)
-
-=item * precision => $precision (optional)
-
-=back
-
 =head2 set_type ($type)
 
 Sets the column's type.
@@ -413,8 +290,8 @@ L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
 =head2 set_attributes (@attributes)
 
 Sets the column's attributes.  These are strings describing the column
-(for example, valid attributes in MySQL are "PRIMARY KEY" or
-"AUTO_INCREMENT").
+(for example, valid attributes in MySQL are 'PRIMARY KEY' or
+'AUTO_INCREMENT').
 
 =head2 add_attribute ($attribute)
 
@@ -480,10 +357,4 @@ Sets the
 L<C<Alzabo::Create::ColumnDefinition>|Alzabo::Create::ColumnDefinition>
 object which holds this column's type information.
 
-=for pod_merge comment
-
-=head2 set_comment ($comment)
-
-Set the comment for the column object.
-
 =cut
