@@ -4,15 +4,10 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Create;
-use Alzabo::Exceptions ( abbr => 'params_exception' );
-
-use Params::Validate qw( :all );
-Params::Validate::validation_options
-    ( on_fail => sub { params_exception join '', @_ } );
 
 use base qw(Alzabo::Index);
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -20,28 +15,18 @@ sub new
 {
     my $proto = shift;
     my $class = ref $proto || $proto;
-
-    validate( @_, { table    => { isa => 'Alzabo::Create::Table' },
-		    columns  => { type => ARRAYREF },
-		    unique   => { type => BOOLEAN, default => 0 },
-		    fulltext => { type => BOOLEAN, default => 0 },
-		    function => { type => UNDEF | SCALAR,  default => undef },
-		  } );
     my %p = @_;
 
     my $self = bless {}, $class;
 
     $self->{table} = $p{table};
     $self->{unique} = $p{unique} || 0;
-    $self->{fulltext} = $p{fulltext} || 0;
-    $self->{function} = $p{function};
 
     $self->{columns} = Tie::IxHash->new;
 
     foreach my $c (@{ $p{columns} })
     {
-	my %p = UNIVERSAL::isa( $c, 'Alzabo::Column' ) ? ( column => $c ) : %$c;
-	$self->add_column(%p);
+	$self->add_column(%$c);
     }
 
     $self->table->schema->rules->validate_index($self);
@@ -51,16 +36,12 @@ sub new
 
 sub add_column
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Create::Column' },
-		    prefix => { type => SCALAR,
-				optional => 1 } } );
+    my Alzabo::Create::Index $self = shift;
     my %p = @_;
 
     my $new_name = $p{column}->name;
 
-    params_exception "Column $new_name already exists in index."
+    Alzabo::Exception::Params->throw( "Column $new_name already exists in index." )
 	if $self->{columns}->EXISTS($new_name);
 
     $self->{columns}->STORE( $new_name, \%p );
@@ -70,19 +51,16 @@ sub add_column
     if ($@)
     {
 	$self->{columns}->DELETE($new_name);
-
-        rethrow_exception($@);
+	$@->rethrow;
     }
 }
 
 sub delete_column
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Column' } );
+    my Alzabo::Create::Index $self = shift;
     my $c = shift;
 
-    params_exception "Column " . $c->name . " is not part of index."
+    Alzabo::Exception::Params->throw( error => "Column " . $c->name . " is not part of index." )
 	unless $self->{columns}->EXISTS( $c->name );
 
     $self->{columns}->DELETE( $c->name );
@@ -90,13 +68,10 @@ sub delete_column
 
 sub set_prefix
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Create::Column' },
-		    prefix => { type => SCALAR } } );
+    my Alzabo::Create::Index $self = shift;
     my %p = @_;
 
-    params_exception "Column " . $p{column}->name . " is not part of index."
+    Alzabo::Exception::Params->throw( error => "Column " . $p{column}->name . " is not part of index." )
 	unless $self->{columns}->EXISTS( $p{column}->name );
 
     my $col = $self->{columns}->FETCH( $p{column}->name );
@@ -115,33 +90,20 @@ sub set_prefix
 	{
 	    delete $col->{prefix};
 	}
-
-        rethrow_exception($@);
+	$@->rethrow;
     }
 }
 
 sub set_unique
 {
-    my $self = shift;
+    my Alzabo::Create::Index $self = shift;
 
-    validate_pos( @_, 1 );
     $self->{unique} = shift;
 }
 
-sub set_fulltext
-{
-    my $self = shift;
-
-    validate_pos( @_, 1 );
-    $self->{fulltext} = shift;
-}
-
 sub register_column_name_change
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Create::Column' },
-		    old_name => { type => SCALAR } } );
+    my Alzabo::Create::Index $self = shift;
     my %p = @_;
 
     return unless $self->{columns}->EXISTS( $p{old_name} );
@@ -184,28 +146,18 @@ C<Alzabo::Index>
 
 The table that this index is indexing.
 
-=item * columns => [ C<Alzabo::Create::Column> object, .. ]
-
 =item * columns => [ { column => C<Alzabo::Create::Column> object,
                        prefix => $prefix },
                       repeat as needed ...
                    ]
 
-This parameter indicates which columns that are being indexed.  It can
-either be an array reference of column objects, or an array reference
-of hash references, each with a key called column and one called
-prefix.
-
-The prefix key is optional.
+This is a list of columns that are being indexes.  The prefix part of
+the hashref is optional.
 
 =item * unique => $boolean
 
 Indicates whether or not this is a unique index.
 
-=item * fulltext => $boolean
-
-Indicates whether or not this is a fulltext index.
-
 =back
 
 =head3 Returns
@@ -254,12 +206,6 @@ Delete the given column from the index.
 
 Set whether or not the index is a unique index.
 
-=for pod_merge fulltext
-
-=head2 set_fulltext ($boolean)
-
-Set whether or not the index is a fulltext index.
-
 =head2 register_column_name_change
 
 =head3 Parameters
