@@ -4,17 +4,12 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Create;
-use Alzabo::Exceptions ( abbr => 'params_exception' );
-
-use Params::Validate qw( :all );
-Params::Validate::validation_options
-    ( on_fail => sub { params_exception join '', @_ } );
 
 use Tie::IxHash;
 
 use base qw(Alzabo::Table);
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.31 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -22,33 +17,20 @@ sub new
 {
     my $proto = shift;
     my $class = ref $proto || $proto;
-
-    validate( @_, { schema => { isa => 'Alzabo::Create::Schema' },
-		    name => { type => SCALAR },
-                    attributes => { type => ARRAYREF,
-                                    optional => 1 },
-		    comment => { type => UNDEF | SCALAR,
-				 default => '' },
-		  } );
     my %p = @_;
 
     my $self = bless {}, $class;
 
     $self->{schema} = $p{schema};
 
+    Alzabo::Exception::Params->throw( error => "No name provided for new table" )
+	unless exists $p{name};
     $self->set_name($p{name});
 
     $self->{columns} = Tie::IxHash->new;
-    $self->{pk} = [];
+    $self->{pk} = Tie::IxHash->new;
     $self->{indexes} = Tie::IxHash->new;
 
-    my %attr;
-    tie %{ $self->{attributes} }, 'Tie::IxHash';
-
-    $self->set_attributes( @{ $p{attributes} } );
-
-    $self->set_comment( $p{comment} );
-
     # Setting this prevents run time type errors.
     $self->{fk} = {};
 
@@ -57,39 +39,22 @@ sub new
 
 sub set_name
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Create::Table $self = shift;
     my $name = shift;
 
-    params_exception "Table $name already exists in schema"
-        if $self->schema->has_table($name);
-
-    my @i;
-    if ($self->{indexes})
-    {
-	@i = $self->indexes;
-	$self->delete_index($_) foreach @i;
-    }
-
     my $old_name = $self->{name};
     $self->{name} = $name;
-
     eval
     {
 	$self->schema->rules->validate_table_name($self);
     };
-
-    $self->add_index($_) foreach @i;
-
+    $self->{name} = $old_name if $@;
     if ($@)
     {
-	$self->{name} = $old_name;
-
-        rethrow_exception($@);
+	$@->rethrow;
     }
 
-    if ( $old_name && eval { $self->schema->table($old_name) } )
+    if ( $old_name && $self->schema->table($old_name) )
     {
 	$self->schema->register_table_name_change( table => $self,
 						   old_name => $old_name );
@@ -104,19 +69,14 @@ sub set_name
 
 sub make_column
 {
-    my $self = shift;
+    my Alzabo::Create::Table $self = shift;
     my %p = @_;
 
     my $is_pk = delete $p{primary_key};
 
-    my %p2;
-    foreach ( qw( before after ) )
-    {
-	$p2{$_} = delete $p{$_} if exists $p{$_};
-    }
     $self->add_column( column => Alzabo::Create::Column->new( table => $self,
 							      %p ),
-		       %p2 );
+		       %p );
 
     my $col = $self->column( $p{name} );
     $self->add_primary_key($col) if $is_pk;
@@ -126,16 +86,12 @@ sub make_column
 
 sub add_column
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Create::Column' },
-		    before => { optional => 1 },
-		    after  => { optional => 1 } } );
+    my Alzabo::Create::Table $self = shift;
     my %p = @_;
 
     my $col = $p{column};
 
-    params_exception "Column " . $col->name . " already exists in " . $self->name
+    Alzabo::Exception::Params->throw( error => "Column " . $col->name . " already exists" )
 	if $self->{columns}->EXISTS( $col->name );
 
     $col->set_table($self) unless $col->table eq $self;
@@ -147,7 +103,7 @@ sub add_column
 	if ( exists $p{$_} )
 	{
 	    $self->move_column( $_ => $p{$_},
-				column => $col );
+				column => $self->column( $p{name} ) );
 	    last;
 	}
     }
@@ -155,12 +111,10 @@ sub add_column
 
 sub delete_column
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Column' } );
+    my Alzabo::Create::Table $self = shift;
     my $col = shift;
 
-    params_exception"Column $col doesn't exist in $self->{name}"
+    Alzabo::Exception::Params->throw( error => "Column $col doesn't exist in $self->{name}" )
 	unless $self->{columns}->EXISTS( $col->name );
 
     $self->delete_primary_key($col) if $col->is_primary_key;
@@ -169,8 +123,7 @@ sub delete_column
     {
 	$self->delete_foreign_key($fk);
 
-	foreach my $other_fk ($fk->table_to->foreign_keys( table => $self,
-                                                           column => $fk->columns_to ) )
+	foreach my $other_fk ($fk->table_to->foreign_keys_by_column( $fk->column_to ) )
 	{
 	    $fk->table_to->delete_foreign_key( $other_fk );
 	}
@@ -186,37 +139,28 @@ sub delete_column
 
 sub move_column
 {
-    my $self = shift;
-
-    validate( @_, { column  => { isa => 'Alzabo::Create::Column' },
-		    before  => { isa => 'Alzabo::Create::Column',
-				 optional => 1 },
-		    after   => { isa => 'Alzabo::Create::Column',
-				 optional => 1 } } );
+    my Alzabo::Create::Table $self = shift;
     my %p = @_;
 
     if ( exists $p{before} && exists $p{after} )
     {
-	params_exception
-            "move_column method cannot be called with both 'before' and 'after' parameters";
+	Alzabo::Exception::Params->throw( error => "move_column method cannot be called with both 'before' and 'after parameters'" );
     }
 
     if ( exists $p{before} )
     {
-	params_exception "Column " . $p{before}->name . " doesn't exist in schema"
+	Alzabo::Exception::Params->throw( error => "Column " . $p{before}->name . " doesn't exist in schema" )
 	    unless $self->{columns}->EXISTS( $p{before}->name );
     }
     else
     {
-	params_exception "Column " . $p{after}->name . " doesn't exist in schema"
+	Alzabo::Exception::Params->throw( error => "Column " . $p{after}->name . " doesn't exist in schema" )
 	    unless $self->{columns}->EXISTS( $p{after}->name );
     }
 
-    params_exception "Column " . $p{column}->name . " doesn't exist in schema"
+    Alzabo::Exception::Params->throw( error => "Column " . $p{column}->name . " doesn't exist in schema" )
 	unless $self->{columns}->EXISTS( $p{column}->name );
 
-    my @pk = $self->primary_key;
-
     $self->{columns}->DELETE( $p{column}->name );
 
     my $index;
@@ -230,130 +174,91 @@ sub move_column
     }
 
     $self->{columns}->Splice( $index, 0, $p{column}->name => $p{column} );
-
-    $self->{pk} = [ $self->{columns}->Indices( map { $_->name } @pk ) ];
 }
 
 sub add_primary_key
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Column' } );
+    my Alzabo::Create::Table $self = shift;
     my $col = shift;
 
     my $name = $col->name;
-    params_exception "Column $name doesn't exist in $self->{name}"
-	unless $self->{columns}->EXISTS($name);
+    Alzabo::Exception::Params->throw( error => "Column $name doesn't exist in $self->{name}" )
+	unless exists $self->{columns}{$name};
 
-    params_exception "Column $name is already a primary key"
-        if $col->is_primary_key;
+    Alzabo::Exception::Params->throw( error => "Column $name is already a primary key" )
+	if $self->{pk}->EXISTS($name);
 
     $self->schema->rules->validate_primary_key($col);
 
-    $col->set_nullable(0);
+    $col->set_null(0);
 
-    my $idx = $self->{columns}->Indices($name);
-    push @{ $self->{pk} }, $idx;
+    $self->{pk}->STORE( $name, $col );
 }
 
 sub delete_primary_key
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Column' } );
+    my Alzabo::Create::Table $self = shift;
     my $col = shift;
 
     my $name = $col->name;
-    params_exception "Column $name doesn't exist in $self->{name}"
+    Alzabo::Exception::Params->throw( error => "Column $name doesn't exist in $self->{name}" )
 	unless $self->{columns}->EXISTS($name);
 
-    params_exception "Column $name is not a primary key"
-        unless $col->is_primary_key;
+    Alzabo::Exception::Params->throw( error => "Column $name is not a primary key" )
+	unless $self->{pk}->EXISTS($name);
 
-    my $idx = $self->{columns}->Indices($name);
-    $self->{pk} = [ grep { $_ != $idx } @{ $self->{pk} } ];
+    $self->{pk}->DELETE($name);
 }
 
 sub make_foreign_key
 {
-    my $self = shift;
+    my Alzabo::Create::Table $self = shift;
 
     $self->add_foreign_key( Alzabo::Create::ForeignKey->new( @_ ) );
 }
 
 sub add_foreign_key
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::ForeignKey' } );
+    my Alzabo::Create::Table $self = shift;
     my $fk = shift;
 
-    foreach my $c ( $fk->columns_from )
-    {
-	push @{ $self->{fk}{ $fk->table_to->name }{ $c->name } }, $fk;
-    }
-
-    if ( ( $fk->is_one_to_one || $fk->is_one_to_many )
-         && !
-         ( $self->primary_key_size == grep { $_->is_primary_key } $fk->columns_from )
-       )
-    {
-        my $i = Alzabo::Create::Index->new( table   => $self,
-                                            columns => [ $fk->columns_from ],
-                                            unique  => 1 );
-
-        # could already have a non-unique index (grr, index id()
-        # method is somewhat broken)
-        $self->delete_index($i) if $self->has_index( $i->id );
-        $self->add_index($i);
-    }
+    push @{ $self->{fk}{ $fk->table_to->name }{ $fk->column_from->name } }, $fk;
 }
 
 sub delete_foreign_key
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::ForeignKey' } );
+    my Alzabo::Create::Table $self = shift;
     my $fk = shift;
 
-    foreach my $c ( $fk->columns_from )
-    {
-	params_exception "Column " . $c->name . " doesn't exist in $self->{name}"
-	    unless $self->{columns}->EXISTS( $c->name );
-    }
+    Alzabo::Exception::Params->throw( error => "Column " . $fk->column_from->name . " doesn't exist in $self->{name}" )
+	unless exists $self->{columns}{ $fk->column_from->name };
 
-    params_exception
-        "No foreign keys to " . $fk->table_to->name . " exist in $self->{name}"
-            unless exists $self->{fk}{ $fk->table_to->name };
+    Alzabo::Exception::Params->throw( error => "No foreign keys to " . $fk->table_to->name . " exist in $self->{name}" )
+	unless exists $self->{fk}{ $fk->table_to->name };
 
+    Alzabo::Exception::Params->throw( error => "Column " . $fk->column_from->name . " is not a foreign key to " . $fk->table_to->name . " in $self->{name}" )
+	unless exists $self->{fk}{ $fk->table_to->name }{ $fk->column_from->name };
+
+    my @current_fk = @{ $self->{fk}{ $fk->table_to->name }{ $fk->column_from->name } };
     my @new_fk;
-    foreach my $c ( $fk->columns_from )
+    foreach my $current_fk (@current_fk)
     {
-	params_exception
-            "Column " . $c->name . " is not a foreign key to " .
-            $fk->table_to->name . " in $self->{name}"
-                unless exists $self->{fk}{ $fk->table_to->name }{ $c->name };
-
-	foreach my $current_fk ( @{ $self->{fk}{ $fk->table_to->name }{ $c->name } } )
+	unless ($fk eq $current_fk)
 	{
-	    push @new_fk, $current_fk unless $current_fk eq $fk;
+	    push @new_fk, $current_fk;
 	}
     }
-
-    foreach my $c ( $fk->columns_from )
+    if (@new_fk)
     {
-	if (@new_fk)
-	{
-	    $self->{fk}{ $fk->table_to->name }{ $c->name } = \@new_fk;
-	}
-	else
-	{
-	    delete $self->{fk}{ $fk->table_to->name }{ $c->name };
-	}
+	$self->{fk}{ $fk->table_to->name }{ $fk->column_from->name } = \@new_fk;
+    }
+    else
+    {
+	delete $self->{fk}{ $fk->table_to->name }{ $fk->column_from->name };
     }
 
     delete $self->{fk}{ $fk->table_to->name }
-	unless keys %{ $self->{fk}{ $fk->table_to->name } };
+	unless keys %{  $self->{fk}{ $fk->table_to->name } };
 }
 
 sub make_index
@@ -367,15 +272,12 @@ sub make_index
 sub add_index
 {
     my Alzabo::Table $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Index' } );
     my $i = shift;
 
-    my $id = $i->id;
-    params_exception "Index already exists (id $id)."
-	if $self->{indexes}->EXISTS($id);
+    Alzabo::Exception::Params->throw( error => "Index already exists." )
+	if $self->{indexes}->EXISTS( $i->id );
 
-    $self->{indexes}->STORE( $id, $i );
+    $self->{indexes}->STORE( $i->id, $i );
 
     return $i;
 }
@@ -383,11 +285,9 @@ sub add_index
 sub delete_index
 {
     my Alzabo::Table $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Create::Index' } );
     my $i = shift;
 
-    params_exception "Index does not exist."
+    Alzabo::Exception::Params->throw( error => "Index does not exist." )
 	unless $self->{indexes}->EXISTS( $i->id );
 
     $self->{indexes}->DELETE( $i->id );
@@ -395,10 +295,7 @@ sub delete_index
 
 sub register_table_name_change
 {
-    my $self = shift;
-
-    validate( @_, { table => { isa => 'Alzabo::Create::Table' },
-		    old_name => { type => SCALAR } } );
+    my Alzabo::Create::Table $self = shift;
     my %p = @_;
 
     $self->{fk}{ $p{table}->name } = delete $self->{fk}{ $p{old_name} }
@@ -407,10 +304,7 @@ sub register_table_name_change
 
 sub register_column_name_change
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Create::Column' },
-		    old_name => { type => SCALAR } } );
+    my Alzabo::Create::Table $self = shift;
     my %p = @_;
 
     my $new_name = $p{column}->name;
@@ -430,67 +324,14 @@ sub register_column_name_change
 	$i->register_column_name_change(%p);
 	$self->add_index($i);
     }
-}
 
-sub set_attributes
-{
-    my $self = shift;
-
-    validate_pos( @_, ( { type => SCALAR } ) x @_ );
-
-    %{ $self->{attributes} } = ();
-
-    foreach (@_)
+    if ( $self->{pk}->EXISTS( $p{old_name} ) )
     {
-	$self->add_attribute($_);
+	my $index = $self->{pk}->Indices( $p{old_name} );
+	$self->{pk}->Replace( $index, $p{column}, $new_name );
     }
 }
 
-sub add_attribute
-{
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
-    my $attr = shift;
-
-    $attr =~ s/^\s+//;
-    $attr =~ s/\s+$//;
-
-    $self->schema->rules->validate_table_attribute( table     => $self,
-                                                    attribute => $attr );
-
-    $self->{attributes}{$attr} = 1;
-}
-
-sub delete_attribute
-{
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
-    my $attr = shift;
-
-    params_exception "Table " . $self->name . " doesn't have attribute $attr"
-	unless exists $self->{attributes}{$attr};
-
-    delete $self->{attributes}{$attr};
-}
-
-sub set_comment { $_[0]->{comment} = defined $_[1] ? $_[1] : '' }
-
-sub save_current_name
-{
-    my $self = shift;
-
-    $self->{last_instantiated_name} = $self->name;
-
-    foreach my $column ( $self->columns )
-    {
-        $column->save_current_name;
-    }
-}
-
-sub former_name { $_[0]->{last_instantiated_name} }
-
 __END__
 
 =head1 NAME
@@ -508,7 +349,7 @@ index, and foreign key objects.
 
 =head1 INHERITS FROM
 
-C<Alzabo::Table>
+Alzabo::Table
 
 =for pod_merge merged
 
@@ -526,10 +367,6 @@ The schema to which this table belongs.
 
 =item * name => $name
 
-=item * comment => $comment
-
-An optional comment.
-
 =back
 
 =head3 Returns
@@ -552,8 +389,6 @@ Changes the name of the table.
 
 =for pod_merge columns
 
-=for pod_merge has_column
-
 =head2 make_column
 
 Creates a new L<C<Alzabo::Create::Column>|Alzabo::Create::Column>
@@ -652,8 +487,6 @@ L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
 
 =for pod_merge primary_key
 
-=for pod_merge primary_key_size
-
 =head2 add_primary_key (C<Alzabo::Create::Column> object)
 
 Make the given column part of the table's primary key.  The primary
@@ -687,10 +520,6 @@ L<C<Alzabo::Create::ForeignKey-E<gt>new>|Alzabo::Create::ForeignKey/new>
 method except for the table parameter, which is automatically added.
 The foreign key object that is created is then added to the table.
 
-If the foreign key being made is 1..1 or 1..n, then a unique index
-will be created on the columns involved in the "1" side of the foreign
-key, unless they are the table's primary key.
-
 =head3 returns
 
 A new L<C<Alzabo::Create::ForeignKey>|Alzabo::Create::ForeignKey>
@@ -710,8 +539,6 @@ L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
 
 =for pod_merge index
 
-=for pod_merge has_index
-
 =for pod_merge indexes
 
 =head2 make_index
@@ -741,34 +568,6 @@ Deletes an index from the table.
 
 L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
 
-=for pod_merge attributes
-
-=for pod_merge has_attribute
-
-=head2 set_attributes (@attributes)
-
-Sets the tables's attributes.  These are strings describing the table
-(for example, valid attributes in MySQL are "TYPE = INNODB" or
-"AUTO_INCREMENT = 100").
-
-=head2 add_attribute ($attribute)
-
-Add an attribute to the column's list of attributes.
-
-=head2 delete_attribute ($attribute)
-
-Delete the given attribute from the column's list of attributes.
-
-=head3 Throws
-
-L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
-
-=for pod_merge comment
-
-=head2 set_comment ($comment)
-
-Set the comment for the table object.
-
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
