@@ -4,13 +4,13 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Exceptions;
+use Alzabo::Util;
 
-use Class::Factory::Util;
 use DBI;
-use Params::Validate qw( validate validate_pos UNDEF SCALAR ARRAYREF );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
 
-$VERSION = 2.0;
+#use fields qw( dbh prepare_method schema );
+
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -26,44 +26,19 @@ sub new
 
     $self->{schema} = $p{schema};
 
+    $self->{prepare_method} ||= 'prepare';
+
     return $self;
 }
 
-sub available { __PACKAGE__->subclasses }
-
-sub _check_dbh
+sub available
 {
-    my $self = shift;
-
-    my $sub = (caller(1))[3];
-    Alzabo::Exception::Driver->throw( error => "Cannot call $sub before calling connect." )
-	unless $self->{dbh};
-}
-
-sub quote
-{
-    my $self = shift;
-
-    $self->_check_dbh;
-
-    return $self->{dbh}->quote(@_);
-}
-
-sub quote_identifier
-{
-    my $self = shift;
-
-    $self->_check_dbh;
-
-    return $self->{dbh}->quote_identifier(@_);
+    return Alzabo::Util::subclasses(__PACKAGE__);
 }
 
 sub rows
 {
     my $self = shift;
-
-    $self->_check_dbh;
-
     my %p = @_;
 
     my $sth = $self->_prepare_and_execute(%p);
@@ -72,7 +47,7 @@ sub rows
     eval
     {
 	my @row;
-	$sth->bind_columns( \ (@row[ 0..$#{ $sth->{NAME_lc} } ] ) );
+	$sth->bind_columns( \ (@row[ 0..$#{ $sth->{NAME} } ] ) );
 
 	push @data, [@row] while $sth->fetch;
 
@@ -86,7 +61,7 @@ sub rows
 					  bind => \@bind );
     }
 
-    return wantarray ? @data : $data[0];
+    return @data;
 }
 
 sub rows_hashref
@@ -94,8 +69,6 @@ sub rows_hashref
     my $self = shift;
     my %p = @_;
 
-    $self->_check_dbh;
-
     my $sth = $self->_prepare_and_execute(%p);
 
     my @data;
@@ -125,8 +98,6 @@ sub one_row
     my $self = shift;
     my %p = @_;
 
-    $self->_check_dbh;
-
     my $sth = $self->_prepare_and_execute(%p);
 
     my @row;
@@ -151,8 +122,6 @@ sub one_row_hash
     my $self = shift;
     my %p = @_;
 
-    $self->_check_dbh;
-
     my $sth = $self->_prepare_and_execute(%p);
 
     my %hash;
@@ -178,15 +147,13 @@ sub column
     my $self = shift;
     my %p = @_;
 
-    $self->_check_dbh;
-
     my $sth = $self->_prepare_and_execute(%p);
 
     my @data;
     eval
     {
 	my @row;
-	$sth->bind_columns( \ (@row[ 0..$#{ $sth->{NAME_lc} } ] ) );
+	$sth->bind_columns( \ (@row[ 0..$#{ $sth->{NAME} } ] ) );
 	push @data, $row[0] while ($sth->fetch);
 	$sth->finish;
     };
@@ -204,21 +171,18 @@ sub column
 sub _prepare_and_execute
 {
     my $self = shift;
-
-    validate( @_, { sql => { type => SCALAR },
-		    bind => { type => UNDEF | SCALAR | ARRAYREF,
-			      optional => 1 } } );
     my %p = @_;
 
-    Alzabo::Exception::Driver->throw( error => "Attempt to access the database without database handle.  Was ->connect called?" )
+    Alzabo::Exception::Driver->throw( error => "Attempt to access the database without database handle.  Was ->connect caled?" )
 	unless $self->{dbh};
 
     my @bind = exists $p{bind} ? ( ref $p{bind} ? @{ $p{bind} } : $p{bind} ) : ();
 
+    my $prep = $self->{prepare_method};
     my $sth;
     eval
     {
-	$sth = $self->{dbh}->prepare( $p{sql} );
+	$sth = $self->{dbh}->$prep( $p{sql} );
 	$sth->execute(@bind);
     };
     if ($@)
@@ -236,8 +200,6 @@ sub do
     my $self = shift;
     my %p = @_;
 
-    $self->_check_dbh;
-
     my $sth = $self->_prepare_and_execute(%p);
 
     my $rows;
@@ -261,28 +223,18 @@ sub tables
 {
     my $self = shift;
 
-    $self->_check_dbh;
-
     my @t = eval {  $self->{dbh}->tables; };
     Alzabo::Exception::Driver->throw( error => $@ ) if $@;
 
     return @t;
 }
 
-sub schemas
-{
-    my $self = shift;
-
-    shift()->_virtual;
-}
-
 sub statement
 {
     my $self = shift;
 
-    $self->_check_dbh;
-
     return Alzabo::DriverStatement->new( dbh => $self->{dbh},
+					 prepare_method => $self->{prepare_method},
 					 @_ );
 }
 
@@ -290,24 +242,18 @@ sub func
 {
     my $self = shift;
 
-    $self->_check_dbh;
-
     my @r;
-    eval
+    if (wantarray)
     {
-	if (wantarray)
-	{
-	    @r = $self->{dbh}->func(@_);
-	    return @r;
-	}
-	else
-	{
-	    $r[0] = $self->{dbh}->func(@_);
-	    return $r[0];
-	}
-    };
-    Alzabo::Exception::Driver->throw( error => $self->{dbh}->errstr )
-	if $self->{dbh}->errstr;
+	@r = eval { $self->{dbh}->func(@_); };
+	return @r;
+    }
+    else
+    {
+	$r[0] = eval { $self->{dbh}->func(@_); };
+	return $r[0];
+    }
+    Alzabo::Exception::Driver->throw( error => $@ ) if $@;
 }
 
 sub DESTROY
@@ -319,26 +265,7 @@ sub DESTROY
 sub disconnect
 {
     my $self = shift;
-    $self->{dbh}->disconnect if $self->{dbh};
-    delete $self->{dbh};
-}
-
-sub handle
-{
-    my $self = shift;
-
-    if (@_)
-    {
-	validate_pos( @_, { isa => 'DBI::db' } );
-	$self->{dbh} = shift;
-    }
-
-    return $self->{dbh};
-}
-
-sub rdbms_version
-{
-    shift()->_virtual;
+    $self->{dbh}->disconnect if ref $self->{dbh};
 }
 
 sub connect
@@ -346,11 +273,6 @@ sub connect
     shift()->_virtual;
 }
 
-sub supports_referential_integrity
-{
-    shift()->_virtual;
-}
-
 sub create_database
 {
     shift()->_virtual;
@@ -366,64 +288,19 @@ sub next_sequence_number
     shift()->_virtual;
 }
 
-sub begin_work
+sub start_transaction
 {
-    my $self = shift;
-
-    $self->_check_dbh;
-
-    $self->{tran_count} = 0 unless defined $self->{tran_count};
-
-    $self->{dbh}->begin_work if $self->{dbh}->{AutoCommit};
-
-    $self->{tran_count}++;
+    shift()->_virtual;
 }
 
 sub rollback
 {
-    my $self = shift;
-
-    $self->_check_dbh;
-
-    $self->{tran_count} = undef;
-
-    eval { $self->{dbh}->rollback unless $self->{dbh}->{AutoCommit} };
-
-    Alzabo::Exception::Driver->throw( error => $@ ) if $@;
-
-    $self->{dbh}->{AutoCommit} = 1;
+    shift()->_virtual;
 }
 
-sub commit
+sub finish_transaction
 {
-    my $self = shift;
-
-    $self->_check_dbh;
-
-    my $callee = (caller(1))[3];
-
-    # More commits than begin_tran.  Not correct.
-    if ( defined $self->{tran_count} )
-    {
-	$self->{tran_count}--;
-    }
-    else
-    {
-	my $caller = (caller(1))[3];
-	require Carp;
-        Carp::cluck( "$caller called commit without corresponding begin_work call\n" );
-    }
-
-    # Don't actually commit until we reach 'uber-commit'
-    return if $self->{tran_count};
-
-    unless ( $self->{dbh}->{AutoCommit} )
-    {
-        $self->{dbh}->commit;
-    }
-    $self->{dbh}->{AutoCommit} = 1;
-
-    $self->{tran_count} = undef;
+    shift()->_virtual;
 }
 
 sub get_last_id
@@ -451,45 +328,48 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Exceptions;
+use Alzabo::Util;
 
 use DBI;
 
-use Params::Validate qw( validate UNDEF SCALAR ARRAYREF );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
+use fields qw( dbh sth sql bind );
 
 $VERSION = '0.1';
 
+1;
+
 sub new
 {
     my $proto = shift;
     my $class = ref $proto || $proto;
-
-    my %p = validate( @_, { dbh   => { can => 'prepare' },
-			    sql   => { type => SCALAR },
-			    bind  => { type => SCALAR | ARRAYREF,
-				       optional => 1 },
-			    limit => { type => UNDEF | ARRAYREF,
-				       optional => 1 } } );
+    my %p = @_;
 
     my $self = bless {}, $class;
 
+    my $prep = $p{prepare_method};
+
+    Alzabo::Exception::Params->throw( error => "Alzabo::DriverStatement->new called with 'dbh' parameter" )
+	unless $p{dbh};
+
     $self->{limit} = $p{limit} ? $p{limit}[0] : 0;
     $self->{offset} = $p{limit} && $p{limit}[1] ? $p{limit}[1] : 0;
     $self->{rows_fetched} = 0;
 
+    $self->{dbh} = $p{dbh};
+
+    $self->{sql} = $p{sql};
     eval
     {
-	$self->{sth} = $p{dbh}->prepare( $p{sql} );
+	$self->{sth} = $self->{dbh}->$prep( $p{sql} );
 
 	$self->{bind} = exists $p{bind} ? ( ref $p{bind} ? $p{bind} : [ $p{bind} ] ) : [];
+	$self->{sth}->execute( @{ $self->{bind} } );
     };
 
     Alzabo::Exception::Driver->throw( error => $@,
-				      sql => $p{sql},
+				      sql => $self->{sql},
 				      bind => $self->{bind} ) if $@;
 
-    $self->execute;
-
     return $self;
 }
 
@@ -497,109 +377,66 @@ sub execute
 {
     my $self = shift;
 
+    $self->{bind} = [@_];
     eval
     {
 	$self->{sth}->finish if $self->{sth}->{Active};
 	$self->{rows_fetched} = 0;
-	$self->{sth}->execute( @_ ? @_ : @{ $self->{bind} } );
-
-        $self->{result} = [];
-        $self->{count} = 0;
-
-        $self->{sth}->bind_columns
-            ( \ ( @{ $self->{result} }[ 0..$#{ $self->{sth}->{NAME_lc} } ] ) );
+	$self->{sth}->execute(@_);
     };
     Alzabo::Exception::Driver->throw( error => $@,
-				      sql => $self->{sth}{Statement},
-				      bind => $self->{bind} ) if $@;
+			 sql => $self->{sql},
+			 bind => $self->{bind} ) if $@;
 }
 
-sub next
-{
-    my $self = shift;
-    my %p = @_;
-
-    return unless $self->{sth}->{Active};
-
-    my $active;
-    eval
-    {
-	do
-	{
-	    $active = $self->{sth}->fetch;
-	} while ( $active && $self->{rows_fetched}++ < $self->{offset} );
-    };
-
-    Alzabo::Exception::Driver->throw( error => $@,
-				      sql => $self->{sth}{Statement},
-				      bind => $self->{bind} ) if $@;
-
-    return unless $active;
-
-    $self->{count}++;
-
-    return wantarray ? @{ $self->{result} } : $self->{result}[0];
-}
-
-sub next_as_hash
+sub next_row
 {
     my $self = shift;
 
     return unless $self->{sth}->{Active};
 
+    my @row;
     my $active;
     eval
     {
+	$self->{sth}->bind_columns( \ (@row[ 0..$#{ $self->{sth}->{NAME} } ] ) );
 	do
 	{
 	    $active = $self->{sth}->fetch;
 	} while ( $active && $self->{rows_fetched}++ < $self->{offset} );
+
+	$self->{sth}->finish if $self->{rows_fetched} == $self->{offset} + $self->{limit};
     };
     Alzabo::Exception::Driver->throw( error => $@,
-				      sql => $self->{sth}{Statement},
+				      sql => $self->{sql},
 				      bind => $self->{bind} ) if $@;
 
-    return unless $active;
+    return @row;
+}
+
+sub next_row_hash
+{
+    my $self = shift;
+
+    return unless $self->{sth}->{Active};
 
     my %hash;
-    @hash{ @{ $self->{sth}->{NAME_lc} } } = @{ $self->{result} };
-
-    $self->{count}++;
-
-    return %hash;
-}
-*next_hash = \&next_as_hash;
-
-sub all_rows
-{
-    my $self = shift;
-
-    my @rows;
-
-    while (my @row = $self->next)
+    my $active;
+    eval
     {
-	push @rows, @row > 1 ? \@row : $row[0];
-    }
+	$self->{sth}->bind_columns( \ ( @hash{ @{ $self->{sth}->{NAME_uc} } } ) );
+	do
+	{
+	    $active = $self->{sth}->fetch;
+	} while ( $active && $self->{rows_fetched}++ < $self->{offset} );
 
-    $self->{count} = scalar @rows;
+	$self->{sth}->finish if $self->{rows_fetched} == $self->{offset} + $self->{limit};
+    };
+    Alzabo::Exception::Driver->throw( error => $@,
+				      sql => $self->{sql},
+				      bind => $self->{bind} ) if $@;
 
-    return @rows;
-}
-
-sub all_rows_hash
-{
-    my $self = shift;
-
-    my @rows;
-
-    while (my %h = $self->next_as_hash)
-    {
-	push @rows, \%h;
-    }
-
-    $self->{count} = scalar @rows;
-
-    return @rows;
+    return $active ? %hash : ();
 }
 
 sub bind
@@ -609,19 +446,14 @@ sub bind
     return @{ $self->{bind} };
 }
 
-sub count { $_[0]->{count} }
-
 sub DESTROY
 {
     my $self = shift;
 
-    local $@;
     eval { $self->{sth}->finish if $self->{sth}; };
     Alzabo::Exception::Driver->throw( error => $@ ) if $@;
 }
 
-1;
-
 __END__
 
 =head1 NAME
@@ -692,21 +524,6 @@ details.
 
 L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
 
-=head2 handle ($optional_dbh)
-
-=head3 Parameters
-
-This method takes one optional parameter, a connected DBI handle.  If
-this is given, then this handle is the new handle for the driver.
-
-=head3 Returns
-
-The active database handle.
-
-=head3 Throws
-
-L<C<Alzabo::Exception::Params>|Alzabo::Exceptions>
-
 =head2 Data Retrieval methods
 
 Some of these methods return lists of data (the
@@ -800,78 +617,42 @@ C<$offset> defaults to 0.
 
 A new L<C<Alzabo::DriverStatement>|Alzabo::DriverStatement> handle,
 ready to return data via the
-L<C<Alzabo::DriverStatement-E<gt>next>|Alzabo::DriverStatement/next>
+L<C<Alzabo::DriverStatement-E<gt>next_row>|Alzabo::DriverStatement/next_row>
 or
-L<C<Alzabo::DriverStatement-E<gt>next_as_hash>|Alzabo::DriverStatement/next_as_hash>
+L<C<Alzabo::DriverStatement-E<gt>next_row_hash>|Alzabo::DriverStatement/next_row_hash>
 methods.
 
 =head3 Throws
 
 L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
 
-
-=head2 rdbms_version
-
-=head3 Returns
-
-The version string of the database backend currently in use.
-
-
 =head1 Alzabo::DriverStatement
 
 This class is a wrapper around C<DBI>'s statement handles.  It finishes
 automatically as appropriate so the end user does need not worry about
 doing this.
 
-=head2 next
+=head2 next_row
 
 Use this method in a while loop to fetch all the data from a
 statement.
 
+=head3 Throws
+
+L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
+
 =head3 Returns
 
 An array containing the next row of data for statement or an empty
 list if no more data is available.
 
-=head3 Throws
-
-L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
-
-=head2 next_as_hash
-
-For backwards compatibility, this is also available as C<next_hash>.
+=head2 next_row_hash
 
 =head3 Returns
 
 A hash containing the next row of data for statement or an empty list
-if no more data is available.  All the keys of the hash will be
-lowercased.
-
-=head3 Throws
-
-L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
-
-=head2 all_rows
-
-=head3 Returns
-
-If the select for which this statement is cursor was for a single
-column (or aggregate value), then method returns an array containing
-each B<remaining> value from the database.
-
-Otherwise, it returns an array of array references, each one
-containing a returned row from the database.
-
-=head3 Throws
-
-L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
-
-=head2 all_rows_hash
-
-=head3 Returns
-
-An array of hashes, each hash representing a single row returned from
-the database.  The hash keys are all in lowercase.
+if no more data is available.  All the keys of the hash are in
+uppercase.
 
 =head3 Throws
 
@@ -888,10 +669,6 @@ will be called first.
 
 L<C<Alzabo::Exception::Driver>|Alzabo::Exceptions>
 
-=head2 count
-
-Returns the number of rows returned so far.
-
 =head1 Alzabo::Exception::Driver METHODS
 
 In addition to the methods inherited from
@@ -908,8 +685,7 @@ The SQL statement in use at the time the error occurred, if any.
 
 =head3 Returns
 
-A array reference contaning the bound parameters for the SQL
-statement, if any.
+A list of the the bound parameters for the SQL statement, if any.
 
 =head1 SUBCLASSING Alzabo::Driver
 
@@ -942,9 +718,10 @@ this:
  5:      my %p = @_;
  6:
  7:      my $self = bless {}, $class;
- 8:
- 9:      return $self;
- 10:  }
+ 8:      $self->{prepare_method} = 'prepare';
+ 9:
+ 10:     return $self;
+ 11:  }
 
 The hash %p contains any values passed to the
 C<Alzabo::Driver-E<gt>new> method by its caller.
@@ -953,6 +730,11 @@ Lines 1-7 should probably be copied verbatim into your own C<new>
 method.  Line 5 can be deleted if you don't need to look at the
 parameters.
 
+Line 8 sets an internal hash entry.  This is a string that defines
+which C<DBI> method to use to prepare a statement.  For some databases
+(such as Oracle), it may be advantageous to change this to
+C<prepare_cached>.
+
 Look at the included C<Alzabo::Driver> subclasses for examples.  Feel
 free to contact me for further help if you get stuck.  Please tell me
 what database you're attempting to implement, what its DBD::* driver
@@ -963,7 +745,7 @@ is, and include the code you've written so far.
 The following methods are not implemented in C<Alzabo::Driver> itself
 and must be implemented in a subclass.
 
-=head3 Parameters for the connect, create_database, and drop_database
+=head3 Parameters for the next three methods
 
 =over 4
 
@@ -973,7 +755,7 @@ and must be implemented in a subclass.
 
 =item * host => $hostname
 
-=item * port => $port
+=item * post => $port
 
 =back
 
@@ -1006,19 +788,27 @@ number based on a column object.  For some databases (MySQL, for
 example), the appropriate value is C<undef>.  This is accounted for in
 Alzabo code that calls this method.
 
-=head2 begin_work
+=head2 start_transaction
 
 Notify Alzabo that you wish to start a transaction.
 
+=head3 Parameters
+
+Not yet determined.
+
 =head2 rollback
 
 Rolls back the current transaction.
 
-=head2 commit
+=head2 finish_transaction
 
 Notify Alzabo that you wish to finish a transaction.  This is
 basically the equivalent of calling commit.
 
+=head3 Parameters
+
+Not yet determined.
+
 =head2 get_last_id
 
 =head3 Returns
