@@ -4,12 +4,9 @@ use strict;
 use vars qw($VERSION);
 
 use Alzabo::Exceptions;
+use Alzabo::Util;
 
-use Class::Factory::Util;
-use Params::Validate qw( validate validate_pos );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
-
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.27 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -23,9 +20,10 @@ sub new
     return "Alzabo::RDBMSRules::$p{rdbms}"->new(@_);
 }
 
-sub available { __PACKAGE__->subclasses }
-
-# validation
+sub available
+{
+    return Alzabo::Util::subclasses(__PACKAGE__);
+}
 
 sub validate_schema_name
 {
@@ -52,11 +50,6 @@ sub validate_column_length
     shift()->_virtual;
 }
 
-sub validate_table_attribute
-{
-    shift()->_virtual;
-}
-
 sub validate_column_attribute
 {
     shift()->_virtual;
@@ -78,19 +71,6 @@ sub validate_index
 }
 
 sub type_is_numeric
-{
-    my $self = shift;
-    my $col  = shift;
-
-    return $self->type_is_integer($col) || $self->type_is_floating_point($col);
-}
-
-sub type_is_integer
-{
-    shift()->_virtual;
-}
-
-sub type_is_floating_point
 {
     shift()->_virtual;
 }
@@ -100,80 +80,40 @@ sub type_is_character
     shift()->_virtual;
 }
 
-sub type_is_date
-{
-    shift()->_virtual;
-}
-
-sub type_is_datetime
-{
-    shift()->_virtual;
-}
-
-sub type_is_time
-{
-    shift()->_virtual;
-}
-
 sub type_is_blob
 {
     shift()->_virtual;
 }
 
-sub blob_type
-{
-    shift()->virtual;
-}
-
-# feature probing
-
-sub column_types
-{
-    shift()->_virtual;
-}
-
-sub feature
-{
-    return 0;
-}
-
-sub quote_identifiers { 0 }
-
-sub schema_attributes
-{
-    shift()->_virtual;
-}
-
-sub table_attributes
-{
-    shift()->_virtual;
-}
-
-sub column_attributes
-{
-    shift()->_virtual;
-}
-
 sub schema_sql
 {
     my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Schema' } );
-
     my $schema = shift;
 
     my @sql;
 
-    local $self->{state};
+    $self->_start_sql;
 
     foreach my $t ( $schema->tables )
     {
 	push @sql, $self->table_sql($t);
     }
 
+    $self->_end_sql;
+
     return @sql;
 }
 
+sub _start_sql
+{
+    1;
+}
+
+sub _end_sql
+{
+    1;
+}
+
 sub table_sql
 {
     shift()->_virtual;
@@ -189,21 +129,17 @@ sub index_sql
     my $self = shift;
     my $index = shift;
 
+    my @cols = $index->columns;
+
     my $index_name = $index->id;
-    $index_name = qq|"$index_name"| if $self->quote_identifiers;
 
     my $sql = 'CREATE';
     $sql .= ' UNIQUE' if $index->unique;
     $sql .= " INDEX $index_name ON " . $index->table->name . ' ( ';
 
-    if ( defined $index->function )
-    {
-        $sql .= $index->function;
-    }
-    else
-    {
-        $sql .= join ', ', map { $_->name } $index->columns;
-    }
+    $sql .= join ', ', ( map { my $sql = join '.', $_->table->name, $_->name;
+			       $sql .= '(' . $index->prefix($_) . ')' if $index->prefix($_);
+			       $sql; } @cols );
 
     $sql .= ' )';
 
@@ -219,10 +155,7 @@ sub drop_table_sql
 {
     my $self = shift;
 
-    my $name = shift->name;
-    $name = qq|"$name"| if $self->quote_identifiers;
-
-    return "DROP TABLE $name";
+    return 'DROP TABLE ' . shift->name;
 }
 
 sub drop_column_sql
@@ -253,10 +186,6 @@ sub column_sql_diff
 sub index_sql_diff
 {
     my $self = shift;
-
-    validate( @_, { new => { isa => 'Alzabo::Index' },
-		    old => { isa => 'Alzabo::Index' } } );
-
     my %p = @_;
 
     my $new_sql = $self->index_sql($p{new});
@@ -271,105 +200,21 @@ sub index_sql_diff
     return @sql;
 }
 
+sub foreign_key_sql_diff
+{
+    shift()->_virtual;
+}
+
 sub alter_primary_key_sql
 {
     shift()->_virtual;
 }
 
-sub can_change_table_name
-{
-    1;
-}
-
-sub can_change_column_name
-{
-    1;
-}
-
-sub change_table_name_sql
-{
-    shift()->_virtual;
-}
-
-sub change_column_name_sql
-{
-    shift()->_virtual;
-}
-
-sub recreate_table_sql
-{
-    shift()->_virtual;
-}
-
-=pod
-
 sub reverse_engineer
 {
-    my $self = shift;
-    my $schema = shift;
-
-    my $dbh = $schema->driver->handle;
-
-    foreach my $table ( $dbh->tables )
-    {
-	my $t = $schema->make_table( name => $table );
-
-	$self->reverse_engineer_table($t);
-    }
+    shift()->_virtual;
 }
 
-sub reverse_engineer_table
-{
-    my $self = shift;
-    my $table = shift;
-
-    my $dbh = $table->schema->driver->handle;
-
-    my $sth = $dbh->column_info( undef, $table->schema->name, $table->name, undef );
-
-    while ( my $col_info = $sth->fetchrow_hashref )
-    {
-	use Data::Dumper; warn Dumper $col_info;
-	my %attr = ( name     => $col_info->{COLUMN_NAME},
-		     type     => $col_info->{TYPE_NAME},
-		     nullable => $col_info->{NULLABLE} ? 1 : 0,
-		   );
-
-	$attr{size} =
-	    $col_info->{COLUMN_SIZE} if $col_info->{COLUMN_SIZE};
-
-	$attr{precision} =
-	    $col_info->{DECIMAL_DIGITS} if $col_info->{DECIMAL_DIGITS};
-
-	$attr{default} =
-	    $col_info->{COLUMN_DEF} if defined $col_info->{COLUMN_DEF};
-
-	$attr{comment} =
-	    $col_info->{REMARKS} if defined $col_info->{REMARKS};
-
-	$table->make_column(%attr);
-    }
-
-    $self->reverse_engineer_table_primary_key($table);
-}
-
-sub reverse_engineer_table_primary_key
-{
-    my $self = shift;
-    my $table = shift;
-
-    my $dbh = $table->schema->driver->handle;
-
-    my $sth = $dbh->column_info( undef, $table->schema->name, $table->name );
-
-    while ( my $pk_info = $sth->fetchrow_hashref )
-    {
-	$table->add_primary_key( $table->column( $pk_info->{COLUMN_NAME} ) );
-    }
-}
-
-=cut
-
 sub rules_id
 {
     shift()->_virtual;
@@ -378,134 +223,72 @@ sub rules_id
 sub schema_sql_diff
 {
     my $self = shift;
-
-    validate( @_, { new => { isa => 'Alzabo::Schema' },
-		    old => { isa => 'Alzabo::Schema' } } );
-
     my %p = @_;
+    my $new = $p{new};
+    my $old = $p{old};
 
-    local $self->{state};
+    $self->_start_sql;
 
     my @sql;
-    my %changed_name;
-    foreach my $new_t ( $p{new}->tables )
+    foreach my $new_t ($new->tables)
     {
-        my $old_name = defined $new_t->former_name ? $new_t->former_name : $new_t->name;
-
-	if ( my $old_t = eval { $p{old}->table($old_name) } )
+	if ( my $old_t = eval { $old->table($new_t->name) } )
 	{
-            if ( $old_name ne $new_t->name )
-            {
-                $changed_name{$old_name} = 1;
-
-                if ( $self->can_change_table_name )
-                {
-                    push @sql, $self->change_table_name_sql($new_t);
-                }
-                else
-                {
-                    push @sql, $self->recreate_table_sql( new => $new_t,
-                                                          old => $old_t,
-                                                        );
-
-                    # no need to do more because table will be
-                    # recreated from scratch
-                    next;
-                }
-            }
-
-            push @sql, $self->table_sql_diff( new => $new_t,
-                                              old => $old_t );
+	    push @sql, $self->table_sql_diff( new => $new_t,
+					      old => $old_t );
+	}
+	else
+	{
+	    push @sql, $self->table_sql($new_t);
 	}
-        else
-        {
-            push @sql, $self->table_sql($new_t);
-        }
     }
 
-    foreach my $old_t ( $p{old}->tables )
+    foreach my $old_t ($old->tables)
     {
-	unless ( $changed_name{ $old_t->name } ||
-                 eval { $p{new}->table( $old_t->name ) } )
+	unless ( eval { $new->table( $old_t->name ) } )
 	{
 	    push @sql, $self->drop_table_sql($old_t);
 	}
     }
 
+    $self->_end_sql;
+
     return @sql;
 }
 
 sub table_sql_diff
 {
     my $self = shift;
-
-    validate( @_, { new => { isa => 'Alzabo::Table' },
-		    old => { isa => 'Alzabo::Table' } } );
-
     my %p = @_;
+    my $new = $p{new};
+    my $old = $p{old};
 
     my @sql;
-    foreach my $old_i ( $p{old}->indexes )
+    foreach my $new_c ($new->columns)
     {
-	unless ( eval { $p{new}->index( $old_i->id ) } )
+	if ( my $old_c = eval { $old->column( $new_c->name ) } )
 	{
-	    push @sql, $self->drop_index_sql($old_i)
-		if eval { $p{new}->columns( map { $_->name } $old_i->columns ) } && ! $@;
-	}
-    }
-
-    my %changed_name;
-    foreach my $new_c ( $p{new}->columns )
-    {
-        $changed_name{ $new_c->former_name } = 1
-            if defined $new_c->former_name && $new_c->former_name ne $new_c->name;
-    }
-
-    foreach my $old_c ( $p{old}->columns )
-    {
-	unless ( $changed_name{ $old_c->name } ||
-                 ( my $new_c = eval { $p{new}->column( $old_c->name ) } )
-               )
-	{
-	    push @sql, $self->drop_column_sql( new_table => $p{new},
-					       old => $old_c );
-	}
-    }
-
-    foreach my $new_c ( $p{new}->columns )
-    {
-        my $old_name = defined $new_c->former_name ? $new_c->former_name : $new_c->name;
-
-	if ( my $old_c = eval { $p{old}->column($old_name) } )
-	{
-            if ( $old_name ne $new_c->name )
-            {
-                if ( $self->can_change_column_name )
-                {
-                    push @sql, $self->change_column_name_sql($new_c);
-                }
-                else
-                {
-                    # no need to do more because table will be
-                    # recreated from scratch
-                    return $self->recreate_table_sql( new => $new_c->table,
-                                                      old => $old_c->table,
-                                                    );
-                }
-            }
-
 	    push @sql, $self->column_sql_diff( new => $new_c,
 					       old => $old_c );
 	}
 	else
 	{
-            push @sql, $self->column_sql_add($new_c);
+	    push @sql, $self->column_sql_add($new_c)
 	}
     }
 
-    foreach my $new_i ( $p{new}->indexes )
+    foreach my $old_c ($old->columns)
     {
-	if ( my $old_i = eval { $p{old}->index( $new_i->id ) } )
+	unless ( my $new_c = eval { $new->column( $old_c->name ) } )
+	{
+	    push @sql, $self->drop_column_sql( new_table => $new,
+					       old => $old_c );
+	}
+    }
+
+    foreach my $new_i ($new->indexes)
+    {
+	if ( my $old_i = eval { $old->index( $new_i->id ) } )
 	{
 	    push @sql, $self->index_sql_diff( new => $new_i,
 					      old => $old_i );
@@ -516,86 +299,70 @@ sub table_sql_diff
 	}
     }
 
-    foreach my $new_fk ( $p{new}->all_foreign_keys )
+    foreach my $old_i ($old->indexes)
     {
-        unless ( grep { $new_fk->id eq $_->id } $p{old}->all_foreign_keys )
+	unless ( eval { $new->index( $old_i->id ) } )
 	{
-	    push @sql, $self->foreign_key_sql($new_fk)
+	    push @sql, $self->drop_index_sql($old_i);
 	}
     }
 
-    foreach my $old_fk ( $p{old}->all_foreign_keys )
+    foreach my $new_fk ($new->all_foreign_keys)
     {
-        unless ( grep { $old_fk->id eq $_->id } $p{new}->all_foreign_keys )
+	if ( my @old_fk = eval { $old->foreign_keys( table => $new_fk->table_to,
+						     column => $new_fk->column_from ) } )
 	{
-	    push @sql, $self->drop_foreign_key_sql($old_fk);
+	    foreach my $old_fk (@old_fk)
+	    {
+		if ( $old_fk->column_to->name eq $new_fk->column_to->name )
+		{
+		    push @sql, $self->foreign_key_sql_diff( new => $new_fk,
+							    old => $old_fk );
+		}
+		else
+		{
+		    push @sql, $self->foreign_key_sql($new_fk)
+		}
+	    }
+	}
+    }
+
+    foreach my $old_fk ($old->all_foreign_keys)
+    {
+	unless ( my @new_fk = eval { $new->foreign_keys( table => $old_fk->table_to,
+							 column => $old_fk->column_from ) } )
+	{
+	    foreach my $new_fk (@new_fk)
+	    {
+		push @sql, $self->drop_foreign_key_sql($old_fk)
+		    unless $old_fk->column_to->name eq $new_fk->column_to->name;
+	    }
 	}
     }
 
     my $pk_changed;
-    foreach my $old_pk ( $p{old}->primary_key )
+    foreach my $old_pk ($old->primary_key)
     {
-        next if $changed_name{ $old_pk->name };
-
-        my $new_col = eval { $p{new}->column( $old_pk->name ) };
-        unless ( $new_col && $new_col->is_primary_key )
-        {
-            push @sql, $self->alter_primary_key_sql( new => $p{new},
-                                                     old => $p{old} );
-
+	unless ( eval { $new->column_is_primary_key($old_pk) } )
+	{
+	    push @sql, $self->alter_primary_key_sql( new => $new,
+						     old => $old );
 	    $pk_changed = 1;
-            last;
-        }
+	    last;
+	}
     }
 
     unless ($pk_changed)
     {
-        foreach my $new_pk ( $p{new}->primary_key )
-        {
-            my $old_col = eval { $p{old}->column( $new_pk->name ) };
-
-            next if $new_pk->former_name && $changed_name{ $new_pk->former_name };
-
-            unless ( $old_col && $old_col->is_primary_key )
-            {
-                push @sql, $self->alter_primary_key_sql( new => $p{new},
-                                                         old => $p{old} );
-
-                last;
-            }
-        }
-    }
-
-    my $alter_attributes;
-    foreach my $new_att ( $p{new}->attributes )
-    {
-        unless ( $p{old}->has_attribute( attribute => $new_att, case_sensitive => 1 ) )
-        {
-            $alter_attributes = 1;
-
-            push @sql, $self->change_table_attributes_sql( new => $p{new},
-                                                           old => $p{old},
-                                                         );
-
-            last;
-        }
-    }
-
-    unless ($alter_attributes)
-    {
-        foreach my $old_att ( $p{old}->attributes )
-        {
-            unless ( $p{new}->has_attribute( attribute => $old_att, case_sensitive => 1 ) )
-            {
-                $alter_attributes = 1;
-
-                push @sql, $self->change_table_attributes_sql( new => $p{new},
-                                                               old => $p{old},
-                                                             );
-
-                last;
-            }
-        }
+	foreach my $new_pk ($new->primary_key)
+	{
+	    unless ( eval { $old->column_is_primary_key($new_pk) } )
+	    {
+		push @sql, $self->alter_primary_key_sql( new => $new,
+							 old => $old );
+		last;
+	    }
+	}
     }
 
     return @sql;
@@ -724,39 +491,6 @@ An array of SQL statements.
 The following methods are not implemented in the C<Alzabo::RDBMSRules>
 class itself and must be implemented in its subclasses.
 
-=head2 column_types
-
-=head3 Returns
-
-A list of valid column type specifiers.
-
-=head2 feature ($feature)
-
-Given a string defining a feature, this method indicates whether or
-not the given RDBMS supports that feature.  By default, this method
-always returns false unless overridden in the subclass.
-
-Features that may be asked for:
-
-=over 4
-
-=item * extended_column_types
-
-Column types that must be input directly from a user, as opposed to
-being chosen from a list.  MySQL's ENUM and SET column types are
-examples of such types.
-
-=item * index_column_prefixes
-
-MySQL supports the notion of column prefixes in indexes, allowing you
-to index only a portion of a large text column.
-
-=item * fulltext_indexes
-
-This should be self-explanatory.
-
-=back
-
 =head2 validate_schema_name (C<Alzabo::Schema> object)
 
 =head3 Returns
@@ -791,7 +525,8 @@ L<C<Alzabo::Exception::RDBMSRules>|Alzabo::Exceptions>
 
 =head3 Returns
 
-A canonized version of the type.
+A boolean value indicate whether or not this type is valid for the
+RDBMS.
 
 =head3 Throws
 
@@ -925,6 +660,25 @@ SQL necessary to turn the 'old' one into the 'new' one.
 
 A list of SQL statements.
 
+=head2 foreign_key_sql_diff
+
+=head3 Parameters
+
+=over 4
+
+=item * new => C<Alzabo::Create::ForeignKey> object
+
+=item * old => C<Alzabo::Create::ForeignKey> object
+
+=back
+
+Given two foreign key objects, this method compares them and generates
+the SQL necessary to turn the 'old' one into the 'new' one.
+
+=head3 Returns
+
+A list of SQL statements.
+
 =head2 alter_primary_key_sql
 
 =head3 Parameters
