@@ -7,7 +7,9 @@ use Alzabo::RDBMSRules;
 
 use base qw(Alzabo::RDBMSRules);
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.35 $ =~ /(\d+)\.(\d+)/;
+
+1;
 
 sub new
 {
@@ -22,7 +24,7 @@ sub validate_schema_name
     my $self = shift;
     my $name = shift->name;
 
-    Alzabo::Exception::RDBMSRules->throw( error => "Schema name must be at least one character long" )
+    Alzabo::Exception::RDBMSRules->throw( error => "Schema name must at least one character long" )
 	unless length $name;
 
     # These are characters that are illegal in a dir name.  I'm trying
@@ -42,11 +44,11 @@ sub validate_table_name
     my $self = shift;
     my $name = shift->name;
 
-    Alzabo::Exception::RDBMSRules->throw( error => "Table name must be at least one character long" )
+    Alzabo::Exception::RDBMSRules->throw( error => "Table name must at least one character long" )
 	unless length $name;
-    Alzabo::Exception::RDBMSRules->throw( error => "Table name is too long.  Names must be 64 characters or less." )
+    Alzabo::Exception::RDBMSRules->throw( error => "Table name '$name' is too long.  Names must be 64 characters or less." )
 	if length $name >= 64;
-    Alzabo::Exception::RDBMSRules->throw( error => "Table name must only contain alphanumerics or underscore(_)." )
+    Alzabo::Exception::RDBMSRules->throw( error => "Table name '$name' must only contain alphanumerics or underscore(_)." )
 	if $name =~ /\W/;
 }
 
@@ -55,24 +57,22 @@ sub validate_column_name
     my $self = shift;
     my $name = shift->name;
 
-    Alzabo::Exception::RDBMSRules->throw( error => "Column name must be at least one character long" )
+    Alzabo::Exception::RDBMSRules->throw( error => "Schema name must at least one character long" )
 	unless length $name;
     Alzabo::Exception::RDBMSRules->throw( error => 'Name is too long.  Names must be 64 characters or less.' )
 	if length $name >= 64;
-    Alzabo::Exception::RDBMSRules->throw( error =>
-					  'Name contains characters that are not alphanumeric or the dollar sign ($).' )
-	if $name =~ /[^\w\$]/;
     Alzabo::Exception::RDBMSRules->throw( error =>
 					  'Name contains only digits.  Names must contain at least one alpha character.' )
 	unless $name =~ /[^\W\d]/;
+    Alzabo::Exception::RDBMSRules->throw( error =>
+					  'Name contains characters that are not alphanumeric or the dollar sign ($).' )
+	if $name =~ /[^\w\$]/;
 }
 
 sub validate_column_type
 {
     my $self = shift;
-    my $type = shift;
-
-    $type = 'INTEGER' if uc $type eq 'INT';
+    my $type = uc shift;
 
     # Columns which take no modifiers.
     my %simple_types = map {$_ => 1} ( qw( DATE
@@ -86,6 +86,7 @@ sub validate_column_type
 					   MEDIUMTEXT
 					   LONGBLOB
 					   LONGTEXT
+					   INT
 					   INTEGER
 					   TINYINT
 					   SMALLINT
@@ -103,38 +104,24 @@ sub validate_column_type
 					 ),
 				     );
 
-    return uc $type if $simple_types{uc $type};
+    return if $simple_types{$type};
 
-    return 'DOUBLE' if $type =~ /DOUBLE\s+PRECISION/i;
+    foreach ( qr/DOUBLE\s+PRECISION/,
+	      qr/NATIONAL\s+CHAR/,
+	      qr/NATIONAL\s+VARCHAR/,
+	     )
+    {
+	return if $type =~ /$_/;
+    }
 
-    return 'CHAR' if $type =~ /\A(?:NATIONAL\s+)?CHAR(?:ACTER)?/i;
-    return 'VARCHAR' if $type =~ /\A(?:NATIONAL\s+)?(?:VARCHAR|CHARACTER VARYING)/i;
+    my $list_val = qr{(['"]).*?\1};
+    my $comma_sep_list = qr{\($list_val(?:\s*,\s*$list_val)+?\)};
 
-    my $t = $self->_capitalize_type($type);
-    return $t if $t;
+    return if $type =~ /\A(?:ENUM|SET)\s*$comma_sep_list\z/o;
 
     Alzabo::Exception::RDBMSRules->throw( error => "Unrecognized type: $type" );
 }
 
-sub _capitalize_type
-{
-    my $self = shift;
-    my $type = shift;
-
-    if ( uc substr($type, 0, 4) eq 'ENUM' )
-    {
-	return 'ENUM' . substr($type, 4);
-    }
-    elsif ( uc substr($type, 0, 3) eq 'SET' )
-    {
-	return 'SET' . substr($type, 3);
-    }
-    else
-    {
-	return uc $type;
-    }
-}
-
 sub validate_column_length
 {
     my $self = shift;
@@ -146,7 +133,7 @@ sub validate_column_length
 	Alzabo::Exception::RDBMSRules->throw( error => "Max display value is too long.  Maximum allowed value is 255." )
 	    if defined $column->length && $column->length > 255;
 
-	Alzabo::Exception::RDBMSRules->throw( error => $column->type . " columns cannot have a precision." )
+	Alzabo::Exception::RDBMSRules->throw( error => "Cannot specify a precision value for an integer column." )
 	    if defined $column->precision;
 	return;
     }
@@ -156,7 +143,7 @@ sub validate_column_length
 	if (defined $column->length)
 	{
 	    Alzabo::Exception::RDBMSRules->throw( error => "Max display value is too long.  Maximum allowed value is 255." )
-		if $column->length > 255;
+		if $1 > 255;
 
 	    Alzabo::Exception::RDBMSRules->throw( error => "Max display value specified without floating point precision." )
 		unless defined $column->precision;
@@ -185,64 +172,49 @@ sub validate_column_length
     {
 	Alzabo::Exception::RDBMSRules->throw( error => "Max display value is too long.  Maximum allowed value is 14." )
 	    if defined $column->length && $column->length > 14;
-	Alzabo::Exception::RDBMSRules->throw( error => $column->type . " columns cannot have a precision." )
-	    if defined $column->precision;
 	return;
     }
 
     if ( $column->type =~ /\A(?:(?:NATIONAL\s+)?VAR)?CHAR/i )
     {
-	Alzabo::Exception::RDBMSRules->throw( error => "CHAR and VARCHAR columns must have a length provided." )
-	    unless defined $column->length && $column->length > 0;
 	Alzabo::Exception::RDBMSRules->throw( error => "Max display value is too long.  Maximum allowed value is 255." )
-	    if $column->length > 255;
-	Alzabo::Exception::RDBMSRules->throw( error => $column->type . " columns cannot have a precision." )
-	    if defined $column->precision;
+	    if defined $column->length && $column->length > 255;
 	return;
     }
 
     if ( uc $column->type eq 'YEAR' )
     {
-	Alzabo::Exception::RDBMSRules->throw( error => "Valid values for the length specification are 2 or 4." )
+	Alzabo::Exception::RDBMSRules->throw( error => "Valid values for the digit specification are 2 or 4." )
 	    if defined $column->length && ($column->length != 2 && $column->length != 4);
 	return;
     }
 
-    Alzabo::Exception::RDBMSRules->throw( error => $column->type . " columns cannot have a length or precision." )
+    Alzabo::Exception::RDBMSRules->throw( error => $column->type . " columns cannot have a length or precision attribute" )
 	if defined $column->length || defined $column->precision;
 }
 
-# placeholder in case we decide to try to do something better later
-sub validate_table_attribute { 1 }
-
 sub validate_column_attribute
 {
     my $self = shift;
     my %p = @_;
 
     my $column = $p{column};
+    my $type = $column->type;
     my $a = uc $p{attribute};
     $a =~ s/\A\s//;
     $a =~ s/\s\z//;
 
-    if ( $a eq 'UNSIGNED' || $a eq 'ZEROFILL' )
+    if ( $a eq 'AUTO_INCREMENT' || $a eq 'UNSIGNED' || $a eq 'ZEROFILL' )
     {
 	Alzabo::Exception::RDBMSRules->throw( error => "$a attribute can only be applied to numeric columns" )
-	    unless $column->is_numeric;
-	return;
-    }
-
-    if ( $a eq 'AUTO_INCREMENT' )
-    {
-	Alzabo::Exception::RDBMSRules->throw( error => "$a attribute can only be applied to integer columns" )
-	    unless $column->is_integer;
+	    unless $self->type_is_numeric($type);
 	return;
     }
 
     if ($a eq 'BINARY')
     {
 	Alzabo::Exception::RDBMSRules->throw( error => "$a attribute can only be applied to character columns" )
-	    unless $column->is_character;
+	    unless $self->type_is_char($type);
 	return;
     }
     return if $a =~ /\AREFERENCES/i;
@@ -264,11 +236,8 @@ sub validate_sequenced_attribute
     my $self = shift;
     my $col = shift;
 
-    Alzabo::Exception::RDBMSRules->throw( error => 'Non-integer columns cannot be sequenced' )
-	unless $col->is_integer;
-
-    Alzabo::Exception::RDBMSRules->throw( error => 'Only one sequenced column per table is allowed.' )
-	if grep { $_ ne $col && $_->sequenced } $col->table->columns;
+    Alzabo::Exception::RDBMSRules->throw( error => 'Non-numeric columns cannot be sequenced' )
+	unless $self->type_is_numeric( $col->type );
 }
 
 sub validate_index
@@ -285,146 +254,46 @@ sub validate_index
 		unless $prefix =~ /\d+/ && $prefix > 0;
 
 	    Alzabo::Exception::RDBMSRules->throw( error => 'Non-character/blob columns cannot have an index prefix' )
-		unless $c->is_blob || $c->is_character;
+		unless $self->type_is_blob( $c->type ) || $self->type_is_char( $c->type );
 	}
 
-	if ( $c->is_blob )
+	if ( $self->type_is_blob( $c->type ) )
 	{
 	    Alzabo::Exception::RDBMSRules->throw( error => 'Blob columns must have an index prefix' )
-		unless $prefix || $index->fulltext;
-	}
-
-	if ( $index->fulltext )
-	{
-	    Alzabo::Exception::RDBMSRules->throw( error => 'A fulltext index can only include text or char columns' )
-		unless $c->is_character || $c->type =~ /\A(?:TINY|MEDIUM|LONG)?TEXT\z/i;
+		unless $prefix;
 	}
     }
-
-    Alzabo::Exception::RDBMSRules->throw( error => 'An fulltext index cannot be unique' )
-	if $index->unique && $index->fulltext;
-
-    Alzabo::Exception::RDBMSRules->throw( error => 'MySQL does not support function indexes' )
-	if defined $index->function;
 }
 
-sub type_is_integer
+sub type_is_numeric
 {
     my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
+    my $type = uc shift;
 
-    return 1 if $type =~ /\A(?:(?:TINY|SMALL|MEDIUM|BIG)?INT|INTEGER)\z/;
-}
-
-sub type_is_floating_point
-{
-    my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
-
-    return 1 if $type =~ /\A(?:FLOAT|DOUBLE|REAL)\z/;
+    return 1 if $type =~ /\A(?:
+                            (?:TINY|SMALL|MEDIUM|BIG)?
+                            INT|INTEGER
+                            )
+                           |
+                           FLOAT|DOUBLE|REAL
+                          \z
+                         /x;
 }
 
 sub type_is_char
 {
     my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
+    my $type = uc shift;
 
-    return 1 if $type =~ /(?:CHAR|TEXT)\z/;
-}
-
-sub type_is_date
-{
-    my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
-
-    return 1 if $type =~ /\A(?:DATE|DATETIME|TIMESTAMP)\z/;
-}
-
-sub type_is_datetime
-{
-    my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
-
-    if ( $type eq 'TIMESTAMP' )
-    {
-        return $col->length > 8;
-    }
-
-    return 1 if $type eq 'DATETIME';
-}
-
-sub type_is_time
-{
-    my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
-
-    if ( $type eq 'TIMESTAMP' )
-    {
-        return $col->length > 8;
-    }
-
-    return 1 if $type eq /\A(?:DATETIME|TIME)\z/;
+    return 1 if $type =~ /\A(?:(?:NATIONAL\s+)?VAR)?CHAR\z/;
 }
 
 sub type_is_blob
 {
     my $self = shift;
-    my $col  = shift;
-    my $type = uc $col->type;
+    my $type = uc shift;
 
-    return 1 if $type =~ /BLOB\z/;
-}
-
-sub blob_type { return 'BLOB' }
-
-sub column_types
-{
-    return qw( TINYINT
-	       SMALLINT
-	       MEDIUMINT
-	       INTEGER
-	       BIGINT
-
-	       FLOAT
-	       DOUBLE
-	       DECIMAL
-	       NUMERIC
-
-	       CHAR
-	       VARCHAR
-
-	       DATE
-	       DATETIME
-	       TIME
-	       TIMESTAMP
-	       YEAR
-
-	       TINYTEXT
-	       TEXT
-	       MEDIUMTEXT
-	       LONGTEXT
-
-	       TINYBLOB
-	       BLOB
-	       MEDIUMBLOB
-	       LONGBLOB
-	     );
-}
-
-my %features = map { $_ => 1 } qw ( extended_column_types
-				    index_prefix
-				    fulltext_index
-				  );
-sub feature
-{
-    shift;
-    return $features{+shift};
+    return 1 if $type =~ /\A(?:TEXT|BLOB)\z/;
 }
 
 sub schema_sql
@@ -456,25 +325,9 @@ sub table_sql
 	$sql .= ",\n";
 	$sql .= '  PRIMARY KEY (';
 	$sql .= join ', ', map {$_->name} @pk;
-	$sql .= ")";
-
-	my @fk = map { $self->foreign_key_sql($_) } $table->all_foreign_keys;
-
-	if (@fk)
-	{
-	    $sql .= ",\n  ";
-	    $sql .= join ",\n  ", @fk;
-	}
-
-	$sql .= "\n";
-    }
-    $sql .= ")";
-
-    if (my @att = $table->attributes)
-    {
-        $sql .= ' ';
-        $sql .= join ' ', @att;
+	$sql .= ')';
     }
+    $sql .= "\n)";
 
     my @sql = ($sql);
     foreach my $i ( $table->indexes )
@@ -489,132 +342,45 @@ sub column_sql
 {
     my $self = shift;
     my $col = shift;
-    my $p = shift; # for skip_name
 
     # make sure each one only happens once
     my %attr = map { uc $_ => $_ } ( $col->attributes,
 				     ($col->nullable ? 'NULL' : 'NOT NULL'),
 				     ($col->sequenced ? 'AUTO_INCREMENT' : () ) );
 
-    # unsigned attribute has to come right after type declaration,
-    # same with binary.  No column could have both.
+    # unsigned attribute has to come right after type declaration
     my @unsigned = $attr{UNSIGNED} ? delete $attr{UNSIGNED} : ();
-    my @binary   = $attr{BINARY} ? delete $attr{BINARY} : ();
 
     my @default;
     if ( defined $col->default )
     {
-	my $def = ( $col->is_numeric ? $col->default :
+	my $def = ( $self->type_is_numeric( $col->type ) ? $col->default :
 		    do { my $d = $col->default; $d =~ s/"/""/g; $d } );
-
 	@default = ( qq|DEFAULT "$def"| );
     }
 
-    my $type = $col->type;
     my @length;
     if ( defined $col->length )
     {
 	my $length = '(' . $col->length;
-	$length .= ', ' . $col->precision if defined $col->precision;
+	$length .= ', ', $col->precision if defined $col->precision;
 	$length .= ')';
-	$type .= $length;
+	@length = $length;
     }
 
-    my @name = $p->{skip_name} ? () : $col->name;
-    my $sql .= join '  ', ( @name,
-			    $type,
+    my $sql .= join '  ', ( $col->name,
+			    $col->type,
+			    @length,
 			    @unsigned,
-			    @binary,
 			    @default,
 			    sort values %attr );
 
     return $sql;
 }
 
-sub index_sql
-{
-    my $self = shift;
-    my $index = shift;
-
-    my $index_name = $self->_make_index_name( $index->id );
-
-    my $sql = 'CREATE';
-    $sql .= ' UNIQUE' if $index->unique;
-    $sql .= ' FULLTEXT' if $index->fulltext;
-    $sql .= " INDEX $index_name ON " . $index->table->name . ' ( ';
-
-    $sql .= join ', ', ( map { my $sql = $_->name;
-			       $sql .= '(' . $index->prefix($_) . ')' if $index->prefix($_);
-			       $sql; } $index->columns );
-
-    $sql .= ' )';
-
-    return $sql;
-}
-
-sub _make_index_name
-{
-    shift;
-    return substr(shift, 0, 64);
-}
-
 sub foreign_key_sql
 {
-    return; # temporarily disable
-    my $self = shift;
-    my $fk = shift;
-
-    return if grep { $_->is_primary_key } $fk->columns_from;
-
-    my $sql = 'FOREIGN KEY ( ';
-    $sql .= join ', ', map { $_->name } $fk->columns_from;
-    $sql .= ' ) REFERENCES ';
-    $sql .= $fk->table_to->name;
-    $sql .= '( ';
-    $sql .= join ', ', map { $_->name } $fk->columns_to;
-    $sql .= ' ) ON DELETE ';
-
-    if ( $fk->from_is_dependent )
-    {
-	$sql .= 'CASCADE';
-    }
-    else
-    {
-	my @to = $fk->columns_to;
-	unless ( ( grep { $_->nullable } @to ) == @to )
-	{
-	    $sql .= 'SET DEFAULT';
-	}
-	else
-	{
-	    $sql .= 'SET NULL';
-	}
-    }
-
-    my $found_index;
-    my @from = $fk->columns_from;
-
- INDEX:
-    foreach my $i ( $fk->table_from->indexes )
-    {
-	my @c = $i->columns;
-
-	next unless @c == @from;
-
-	for ( 0..$#c )
-	{
-	    next INDEX unless $c[$_]->name eq $from[$_]->name;
-	}
-
-	$found_index = 1;
-    }
-
-    unless ($found_index)
-    {
-	$fk->table_from->make_index( columns => [ @from ] );
-    }
-
-    return $sql;
+    return;
 }
 
 sub drop_column_sql
@@ -635,7 +401,7 @@ sub drop_index_sql
     my $self = shift;
     my $index = shift;
 
-    return 'DROP INDEX ' . $self->_make_index_name( $index->id ) . ' ON ' . $index->table->name;
+    return 'DROP INDEX ' . $index->id . ' ON ' . $index->table->name;
 }
 
 sub column_sql_add
@@ -643,20 +409,7 @@ sub column_sql_add
     my $self = shift;
     my $col = shift;
 
-    my $sequenced = 0;
-    if ( ($sequenced = $col->sequenced) )
-    {
-	$col->set_sequenced(0);
-    }
-
-    my $new_sql = $self->column_sql($col);
-
-    if ($sequenced)
-    {
-	$col->set_sequenced(1);
-    }
-
-    return 'ALTER TABLE ' . $col->table->name . ' ADD COLUMN ' . $new_sql;
+    return 'ALTER TABLE ' . $col->table->name . ' ADD COLUMN ' . $self->column_sql($col);
 }
 
 sub column_sql_diff
@@ -666,39 +419,17 @@ sub column_sql_diff
     my $new = $p{new};
     my $old = $p{old};
 
-    my $sequenced = 0;
-    if ( ( $sequenced = $new->sequenced ) && ! $old->sequenced )
-    {
-	$new->set_sequenced(0);
-    }
+    my $new_sql = $self->column_sql($new);
 
-    my $new_sql = $self->column_sql( $new, { skip_name => 1 } );
+    my $sql = 'ALTER TABLE ' . $new->table->name . ' CHANGE COLUMN ' . $new->name . ' ' . $new_sql
+	if $new_sql ne $self->column_sql($old);
 
-    if ($sequenced)
-    {
-	$new->set_sequenced(1);
-    }
+    return $sql || ();
+}
 
-    my @sql;
-    if ( $new_sql ne $self->column_sql( $old, { skip_name => 1 } ) ||
-	 ( $new->sequenced && ! $old->sequenced ) )
-    {
-	my $sql =
-            ( 'ALTER TABLE ' . $new->table->name . ' CHANGE COLUMN ' .
-              $new->name . ' ' . $new->name . ' ' . $new_sql
-            );
-
-	# can't have more than 1 auto_increment column per table (dumb!)
-	if ( ( $new->sequenced && ! $old->sequenced ) &&
-	     ! grep { $_ ne $new && $_->sequenced } $new->table->columns )
-	{
-	    $sql .= ' AUTO_INCREMENT' if $new->sequenced && ! $old->sequenced;
-	}
-
-	push @sql, $sql;
-    }
-
-    return @sql;
+sub foreign_key_sql_diff
+{
+    return;
 }
 
 sub alter_primary_key_sql
@@ -713,7 +444,7 @@ sub alter_primary_key_sql
     push @sql, 'ALTER TABLE ' . $new->name . ' DROP PRIMARY KEY'
 	if $old->primary_key;
 
-    if ( $new->primary_key )
+    if ($new->primary_key)
     {
 	my $sql = 'ALTER TABLE  ' . $new->name . ' ADD PRIMARY KEY ( ';
 	$sql .= join ', ', map {$_->name} $new->primary_key;
@@ -722,59 +453,9 @@ sub alter_primary_key_sql
 	push @sql, $sql;
     }
 
-    foreach ( $new->primary_key )
-    {
-	if ( $_->sequenced &&
-	     ! ( $old->has_column( $_->name ) &&
-		 $old->column( $_->name )->is_primary_key ) )
-	{
-	    my $sql = $self->column_sql($_);
-	    push @sql,
-                'ALTER TABLE ' . $new->name . ' CHANGE COLUMN ' . $_->name . ' ' . $sql;
-	}
-    }
-
     return @sql;
 }
 
-sub change_table_name_sql
-{
-    my $self = shift;
-    my $table = shift;
-
-    return 'RENAME TABLE ' . $table->former_name . ' TO ' . $table->name;
-}
-
-sub change_table_attributes_sql
-{
-    my $self = shift;
-    my %p = @_;
-
-    return 'ALTER TABLE ' . $p{new}->name . ' ' . join ' ', $p{new}->attributes;
-}
-
-sub change_column_name_sql
-{
-    my $self = shift;
-    my $column = shift;
-
-    return
-        ( 'ALTER TABLE ' . $column->table->name . ' CHANGE COLUMN ' .
-          $column->former_name . ' ' . $self->column_sql($column)
-        );
-}
-
-my %ignored_defaults = ( DATETIME => '0000-00-00 00:00:00',
-			 DATE => '0000-00-00',
-			 YEAR => '0000',
-			 CHAR => '',
-			 VARCHAR => '',
-			 TINTYTEXT => '',
-			 SMALLTEXT => '',
-			 TEXT => '',
-			 LONGTEXT => '',
-		       );
-
 sub reverse_engineer
 {
     my $self = shift;
@@ -784,23 +465,12 @@ sub reverse_engineer
 
     foreach my $table ( $driver->tables )
     {
-	( my $table_name = $table ) =~ s/^[`"]|[`"]$//g;
-
-	my $t = $schema->make_table( name => $table_name );
-
+	my $t = $schema->make_table( name => $table );
 	foreach my $row ( $driver->rows( sql => "DESCRIBE $table" ) )
 	{
-            my ($type, @a);
-            if ( $row->[1] =~ /\A(?:ENUM|SET)/i )
-            {
-                $type = $row->[1];
-            }
-            else
-            {
-                ($type, @a) = split /\s+/, $row->[1];
-            }
+	    my ($type, @a) = split /\s+/, $row->[1];
 
-	    my $default = $row->[4] if defined $row->[4] && uc $row->[4] ne 'NULL';
+	    my $default = $row->[4] if $row->[4] && uc $row->[4] ne 'NULL';
 
 	    my $seq = 0;
 	    foreach my $a ( split /\s+/, $row->[5] )
@@ -816,43 +486,31 @@ sub reverse_engineer
 	    }
 
 	    my %p;
-	    if ( $type !~ /ENUM|SET/i
-		 && $type =~ /(\w+)\((\d+)(?:\s*,\s*(\d+))?\)$/ )
+	    if ($type !~ /enum|set/i && $type =~ /(.+)\((\d+)(?:\s*,\s*(\d+))?\)$/)
 	    {
-		$type = uc $1;
-		$type = 'INTEGER' if $type eq 'INT';
+		$type = $1;
 
 		# skip defaults
-		unless ( $type eq 'TINYINT' && ( $2 == 4 || $2 == 3 ) ||
-			 $type eq 'SMALLINT' && ( $2 == 6 || $2 == 5 ) ||
-			 $type eq 'MEDIUMINT' && ( $2 == 6 || $2 == 5 ) ||
-			 $type eq 'INTEGER' && ( $2 == 11 || $2 == 10 )  ||
-			 $type eq 'BIGINT' && ( $2 == 21 || $2 == 20 ) ||
-			 $type eq 'YEAR' && $2 == 4
-		       )
+		unless ( $type eq 'tinyint' && $2 == 4 ||
+			 $type eq 'smallint' && $2 == 6 ||
+			 $type eq 'mediumint' && $2 == 6 ||
+			 $type eq 'int' && $2 == 11 ||
+			 $type eq 'bigint' && $2 == 21 )
 		{
 		    $p{length} = $2;
 		    $p{precision} = $3;
 		}
 	    }
 
-	    $type = $self->_capitalize_type($type);
-
-	    if (defined $default)
-	    {
-		$default = undef
-		    if exists $ignored_defaults{$type} && $default eq $ignored_defaults{$type};
-	    }
-
  	    my $c = $t->make_column( name => $row->[0],
 				     type => $type,
 				     nullable => $row->[2] eq 'YES',
 				     sequenced => $seq,
 				     default => $default,
 				     attributes => \@a,
-				     primary_key => $row->[3] eq 'PRI',
 				     %p,
 				   );
+	    $t->add_primary_key($c) if $row->[3] eq 'PRI';
 	}
 
 	my %i;
@@ -864,17 +522,12 @@ sub reverse_engineer
 	    $i{ $row->[2] }{cols}[ $row->[3] - 1 ]{prefix} = $row->[7]
 		if defined $row->[7];
 	    $i{ $row->[2] }{unique} = $row->[1] ? 0 : 1;
-
-            my $type_i = $driver->major_version >= 4 ? 10 : 9;
-	    $i{ $row->[2] }{fulltext} =
-                $row->[$type_i] && $row->[$type_i] =~ /fulltext/i ? 1 : 0;
 	}
 
 	foreach my $index (keys %i)
 	{
-	    $t->make_index( columns  => $i{$index}{cols},
-			    unique   => $i{$index}{unique},
-			    fulltext => $i{$index}{fulltext} );
+	    $t->make_index( columns => $i{$index}{cols},
+			    unique  => $i{$index}{unique} );
 	}
     }
 }
@@ -884,13 +537,11 @@ sub rules_id
     return 'MySQL';
 }
 
-1;
-
 __END__
 
 =head1 NAME
 
-Alzabo::RDBMSRules::MySQL - MySQL specific database rules.
+Alzabo::RDBMSRules::MySQL - Perl extension for blah blah blah
 
 =head1 SYNOPSIS
 
