@@ -3,15 +3,12 @@ package Alzabo::Runtime::RowCursor;
 use strict;
 use vars qw($VERSION);
 
-use Alzabo::Exceptions;
 use Alzabo::Runtime;
 
-use Params::Validate qw( :all );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
-
 use base qw( Alzabo::Runtime::Cursor );
+use fields qw( table );
 
-$VERSION = 2.0;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -19,20 +16,19 @@ sub new
 {
     my $proto = shift;
     my $class = ref $proto || $proto;
+    my %p = @_;
 
-    my %p = validate( @_, { statement => { isa => 'Alzabo::DriverStatement' },
-			    table => { isa => 'Alzabo::Runtime::Table' },
-			  } );
+    my $self;
 
-    my $self = bless { %p,
-		       seen => {},
-		       count => 0,
-		     }, $class;
+    $self->{statement} = $p{statement};
+    $self->{table} = $p{table};
+    $self->{errors} = [];
+    $self->{no_cache} = $p{no_cache};
 
-    return $self;
+    return bless $self, $class;
 }
 
-sub next
+sub next_row
 {
     my $self = shift;
 
@@ -47,31 +43,33 @@ sub next
     # 1 but no such row actually exists then we want to skip this.
     #
     # If they really want to know we do save the exception.
-    until ( defined $row )
+    do
     {
-	my @row = $self->{statement}->next;
+	$self->{errors} = [];
 
-	last unless @row && grep { defined } @row;
+	my @row = $self->{statement}->next_row;
+
+	return unless @row && grep { defined } @row;
 
 	my %hash;
-	my @pk = $self->{table}->primary_key;
-	@hash{ map { $_->name } @pk } = @row[0..$#pk];
+	@hash{ map { $_->name } $self->{table}->primary_key } = @row;
 
-	my %prefetch;
-	if ( (my @pre = $self->{table}->prefetch) && @row > @pk )
+	$row = eval { $self->{table}->row_by_pk( @_,
+						 pk => \%hash,
+						 no_cache => $self->{no_cache},
+					       ); };
+	if ($@)
 	{
-	    @prefetch{@pre} = @row[$#pk + 1 .. $#row];
+	    if ( $@->isa('Alzabo::Exception::NoSuchRow') )
+	    {
+		push @{ $self->{errors} },  $@;
+	    }
+	    else
+	    {
+		$@->rethrow;
+	    }
 	}
-
-	$row = $self->{table}->row_by_pk( @_,
-                                          pk => \%hash,
-                                          prefetch => \%prefetch,
-                                          %{ $self->{row_params} },
-                                        );
-    }
-
-    return unless $row;
-    $self->{count}++;
+    } until (defined $row);
 
     return $row;
 }
@@ -81,14 +79,14 @@ sub all_rows
     my $self = shift;
 
     my @rows;
-    while ( my $row = $self->next )
+    my @errors;
+    while ( my $row = $self->next_row )
     {
 	push @rows, $row;
+	push @errors, $self->errors if $self->errors;
     }
 
-
-    $self->{count} = scalar @rows;
-
+    $self->{errors} = \@errors;
     return @rows;
 }
 
@@ -104,7 +102,7 @@ Alzabo::Runtime::RowCursor - Cursor that returns C<Alzabo::Runtime::Row> objects
 
   my $cursor = $schema->table('foo')->all_rows;
 
-  while ( my $row = $cursor->next )
+  while ( my $row = $cursor->next_row )
   {
       print $row->select('foo'), "\n";
   }
@@ -136,36 +134,32 @@ L<C<Alzabo::Runtime::Cursor>|Alzabo::Runtime::Cursor>
 
 =back
 
-=head2 next
+=head2 next_row
 
 Returns the next L<C<Alzabo::Runtime::Row>|Alzabo::Runtime::Row>
-object or undef if no more are available.
+object or undef if no more are available.  This behavior can mask
+errors in your database's referential integrity.  For more information
+on how to deal with this see L<the HANDLING ERRORS section in
+Alzabo::Runtime::Cursor|Alzabo::Runtime::Cursor/HANDLING ERRORS>.
 
 =head2 all_rows
 
 Returns all the rows available from the current point onwards.  This
 means that if there are five rows that will be returned when the
-object is created and you call C<next> twice, calling all_rows
-after it will only return three.
+object is created and you call C<next_row> twice, calling all_rows
+after it will only return three.  Calling the L<C<errors>|errors>
+method after this will return all errors trapped during the fetching
+of these rows.
+
+=head2 errors
+
+See L<C<Alzabo::Runtime::Cursor>|Alzabo::Runtime::Cursor>.
 
 =head2 reset
 
-Resets the cursor so that the next L<C<next>|next> call will
+Resets the cursor so that the next L<C<next_row>|next_row> call will
 return the first row of the set.
 
-=head2 count
-
-=head3 Returns
-
-The number of rows returned by the cursor so far.
-
-=head2 next_as_hash
-
-=head3 Returns
-
-The next row in a hash, where the hash key is the table name and the
-hash value is the row object.
-
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
