@@ -5,122 +5,73 @@ use vars qw($VERSION);
 
 use Alzabo;
 
-use Params::Validate qw( :all );
-Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params->throw( error => join '', @_ ) } );
-
 use Tie::IxHash;
 
-$VERSION = 2.0;
+#use fields qw( schema name columns indexes pk fk );
+
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
 sub schema
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
 
     return $self->{schema};
 }
 
 sub name
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
 
     return $self->{name};
 }
 
-sub has_column
-{
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
-
-    return $self->{columns}->FETCH(shift);
-}
-
 sub column
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
     my $name = shift;
 
-    if ( my $col = $self->{columns}->FETCH($name) )
-    {
-        return $col;
-    }
-    else
-    {
-        Alzabo::Exception::Params->throw
-            ( error => "Column $name doesn't exist in $self->{name}" );
-    }
+    Alzabo::Exception::Params->throw( error => "Column $name doesn't exist in $self->{name}" )
+	unless $self->{columns}->EXISTS($name);
+
+    return $self->{columns}->FETCH($name);
 }
 
 sub columns
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
+
+    if (@_)
+    {
+	return map { $self->column($_) } @_;
+    }
 
-    return $self->column(@_) if @_ ==1 ;
-    return map { $self->column($_) } @_ if @_ > 1;
     return $self->{columns}->Values;
 }
 
 sub primary_key
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
 
-    return unless @{ $self->{pk} };
-
-    return ( wantarray ?
-             $self->{columns}->Values( @{ $self->{pk} } ) :
-             $self->{columns}->Values( $self->{pk}[0] )
-           );
-}
-
-sub primary_key_size
-{
-    my $self = shift;
-
-    return scalar @{ $self->{pk} };
+    return $self->{pk}->Values;
 }
 
 sub column_is_primary_key
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Column' } );
-
-    my $name = shift->name;
+    my Alzabo::Table $self = shift;
+    my $col = shift;
 
+    my $name = $col->name;
     Alzabo::Exception::Params->throw( error => "Column $name doesn't exist in $self->{name}" )
 	unless $self->{columns}->EXISTS($name);
 
-    my $idx = $self->{columns}->Indices($name);
-    return 1 if grep { $idx == $_ } @{ $self->{pk} };
-
-    return 0;
-}
-
-sub attributes
-{
-    return keys %{ $_[0]->{attributes} };
-}
-
-sub has_attribute
-{
-    my $self = shift;
-    my %p = validate( @_, { attribute => { type => SCALAR },
-			    case_sensitive => { type => SCALAR,
-						optional => 1 } } );
-
-    my $att = $p{case_sensitive} ? $p{attribute} : lc $p{attribute};
-
-    return exists $self->{attributes}{$att};
+    return $self->{pk}->EXISTS($name);
 }
 
 sub foreign_keys
 {
-    my $self = shift;
-
-    validate( @_, { column => { isa => 'Alzabo::Column' },
-		    table  => { isa => 'Alzabo::Table' } } );
+    my Alzabo::Table $self = shift;
     my %p = @_;
 
     my $c_name = $p{column}->name;
@@ -140,9 +91,7 @@ sub foreign_keys
 
 sub foreign_keys_by_table
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Table' } );
+    my Alzabo::Table $self = shift;
     my $name = shift->name;
 
     my $fk = $self->{fk};
@@ -160,9 +109,7 @@ sub foreign_keys_by_table
 
 sub foreign_keys_by_column
 {
-    my $self = shift;
-
-    validate_pos( @_, { isa => 'Alzabo::Column' } );
+    my Alzabo::Table $self = shift;
     my $col = shift;
 
     Alzabo::Exception::Params->throw( error => "Column " . $col->name . " doesn't exist in $self->{name}" )
@@ -183,21 +130,15 @@ sub foreign_keys_by_column
 
 sub all_foreign_keys
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
 
-    my %seen;
     my @fk;
     my $fk = $self->{fk};
     foreach my $t (keys %$fk)
     {
 	foreach my $c ( keys %{ $fk->{$t} } )
 	{
-	    foreach my $key ( @{ $fk->{$t}{$c} } )
-	    {
-		next if $seen{$key};
-		push @fk, $key;
-		$seen{$key} = 1;
-	    }
+	    push @fk, @{ $fk->{$t}{$c} };
 	}
     }
 
@@ -206,9 +147,7 @@ sub all_foreign_keys
 
 sub index
 {
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
+    my Alzabo::Table $self = shift;
     my $id = shift;
 
     Alzabo::Exception::Params->throw( error => "Index $id doesn't exist in $self->{name}" )
@@ -217,25 +156,13 @@ sub index
     return $self->{indexes}->FETCH($id);
 }
 
-sub has_index
-{
-    my $self = shift;
-
-    validate_pos( @_, { type => SCALAR } );
-    my $id = shift;
-
-    return $self->{indexes}->EXISTS($id);
-}
-
 sub indexes
 {
-    my $self = shift;
+    my Alzabo::Table $self = shift;
 
     return $self->{indexes}->Values;
 }
 
-sub comment { $_[0]->{comment} }
-
 __END__
 
 =head1 NAME
@@ -284,18 +211,9 @@ given.
 
 =head3 Returns
 
-If no arguments are given, returns a list of all
-L<C<Alzabo::Column>|Alzabo::Column> objects in the schema, or in a
-scalar context the number of such tables.  If one or more arguments
-are given, returns a list of table objects with those names, in the
-same order given.
-
-=head2 has_column ($name)
-
-=head3 Returns
-
-A true or false value depending on whether or not the column exists in
-the table.
+A list of L<C<Alzabo::Column>|Alzabo::Column> objects that match the
+list of names given.  If no list is provided, then it returns all
+column objects for the table.
 
 =head2 primary_key
 
@@ -308,15 +226,8 @@ usually matters in indexes.
 
 =head3 Returns
 
-In array context, return an ordered list of column objects that make
-up the primary key for the table.  In scalar context, it returns the
-first element of that list.
-
-=head2 primary_key_size
-
-=head3 Returns
-
-The number of columsn in the table's primary key.
+An ordered list of column objects that make up the primary key for the
+table.
 
 =head2 column_is_primary_key (C<Alzabo::Column> object)
 
@@ -330,35 +241,6 @@ method on the column object.
 A boolean value indicating whether or not the column given is part of
 the table's primary key.
 
-=head2 attributes
-
-A column's attributes are strings describing the column (for example,
-valid attributes in MySQL are 'UNSIGNED' or 'ZEROFILL'.
-
-=head3 Returns
-
-A list of strings.
-
-=head2 has_attribute
-
-=head3 Parameters:
-
-This method can be used to test whether or not a column has a
-particular attribute.  By default, the check is case-insensitive.
-
-=over 4
-
-=item * attribute => $attribute
-
-=item * case_sensitive => 0 or 1 (defaults to 0)
-
-=back
-
-=head3 Returns
-
-A boolean value indicating whether or not the column has this
-particular attribute.
-
 =head2 foreign_keys
 
 =head3 Parameters
@@ -403,7 +285,7 @@ list.  There is no guarantee as to what the first item will be.
 
 =head2 index ($index_id)
 
-This method expects an index id as returned by the
+This method expect an index id as returned by the
 L<C<Alzabo::Index-E<gt>id>|Alzabo::Index/id> method.
 
 =head3 Returns
@@ -411,28 +293,12 @@ L<C<Alzabo::Index-E<gt>id>|Alzabo::Index/id> method.
 The L<C<Alzabo::Index>|Alzabo::Index> object matching this id, if it
 exists in the table.
 
-=head2 has_index ($index_id)
-
-This method expects an index id as returned by the
-L<C<Alzabo::Index-E<gt>id>|Alzabo::Index/id> method.
-
-=head3 Returns
-
-A boolean indicating whether or not the table has an index with the
-same id.
-
 =head2 indexes
 
 =head3 Returns
 
 All the L<C<Alzabo::Index>|Alzabo::Index> objects for the table.
 
-=head2 comment
-
-=head3 Returns
-
-The comment associated with the table object, if any.
-
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
